/** 
 * Calculates and returns path for the animation
 * @method getCircleAnimationPath
 * @param {Strign} object - object in convertor
 * @param {String} type - type of an object
 * @param {Int} direction - direction of the animation, possible values are 0 or 1, if not given direction will be 1
 * @return {String} path - retuns path as a string that has the array in it
 */
function getAnimationPath(object, type, direction) {
  var c = object;
  var path = []; 
  var next_tick = 0;
  var ticksaddon = 0.1;
  if (type == 'circle') {
    while (next_tick <= 1) {
      path.push("["+c+".X("+next_tick+"), "+c+".Y("+next_tick+")]");
      next_tick = Math.round((next_tick + ticksaddon)*10)/10; 
    }
    if (direction == 1) path = path.reverse();
    //path.pop();
  }
  else {
    if (direction == 0) {
      path.push("["+c+".point2.X(),"+c+".point2.Y()]");
      path.push("["+c+".point1.X(),"+c+".point1.Y()]");
      path.push("["+c+".point2.X(),"+c+".point2.Y()]");
    }
    else {
      path.push("["+c+".point1.X(),"+c+".point1.Y()]");
      path.push("["+c+".point2.X(),"+c+".point2.Y()]");
      path.push("["+c+".point1.X(),"+c+".point1.Y()]");
    }
  }

  return "["+path+"]";
}

/** 
 * This function creates label offset for the elements in jsxgboard
 * @method createLabelAlign
 * @param {String} l - value given in UI, direction for moving label
 * @param {String} a - value is how much to move the label
 * @return {String} - attribute to change label position in JSXGraph image 
 */
function createLabelAlign(l, v) {
  
  if (l == "ct") {return "{offset:[0," + (10+v) + "]}"}
  else if (l == "rt") {return "{offset:[10," + (10+v) + "]}"}
  else if (l == "rm") {return "{offset:[10," + (0+v) + "]}"}
  else if (l == "rb") {return "{offset:[10," + (-10+v) + "]}"}
  else if (l == "cb") {return "{offset:[0," + (-10+v) + "]}"}
  else if (l == "lb") {return "{offset:[-10," + (-10+v) + "]}"}
  else if (l == "lm") {return "{offset:[-10," + (0+v) + "]}"}
  else if (l == "lt") {return "{offset:[-10," + (10+v) + "]}"}
  else try {
    //alert("{offset:["+l+"]}");
    return "{offset:["+l+"]}";
  } catch(e){alert(e.message);}
}

function getBoardSizeFromCode(codetxt) {
    var goodChar="0123456789", goodWidth=["WIDTH", "Width", "width"], goodHeight=["HEIGHT", "Height", "height"], indx1=0, indx2=0, w="", h="";

    for (var i=0; i<goodWidth.length; i++) {
      if (codetxt.indexOf(goodWidth[i])>=0) {
        indx1 = codetxt.indexOf(goodWidth[i]);
        while (goodChar.indexOf(codetxt[indx1])==-1)
          indx1++;
        indx2 = indx1;
        while (goodChar.indexOf(codetxt[indx2])>=0)
          indx2++;
        w = codetxt.slice(indx1, indx2);
        indx1=0; indx2=0;
      }
    }
    if (w.lenght<1) alert("Load error. File has no width value.");
    for (var i=0; i<goodHeight.length; i++) {
      if (codetxt.indexOf(goodHeight[i])>=0) {
        indx1 = codetxt.indexOf(goodHeight[i]);
        while (goodChar.indexOf(codetxt[indx1])==-1)
          indx1++;
        indx2 = indx1;
        while (goodChar.indexOf(codetxt[indx2])>=0)
          indx2++;
        h = codetxt.slice(indx1, indx2);
        indx1=0; indx2=0;
      }
    }
    if (h.lenght<1) alert("Load error. File has no height value.");
    return [w,h];  
}

function getOrigString(e, w, h, withtags, options) {
  var s="";
  var com = "", jspcom = "", lastcom = "";

  var end = "\n{ This JavaSketchpad code was generated by JSP to JSXG convertor   }\n{ see http://cs.uef.fi/matematiikka/ABACUS/JavasketcpadToJSXgraph/ }\n";
  
  if (options == 'SketchrowOnly') {
    
    if (e.JSPComment.length == 0 && e.comment.length > 0) s = s + "{" + e.comment.slice(2,this.length-3) + "}"  ;
    s = s + "{"+e.Row+"} " + e.Type + "(";
    s = s+e.Pos + ")";
    s = s + e.Opts + ";\n";
    s = s.replace('))[', ')[');  
    if (e.comment.length > 0 && e.JSPComment.length > 0) s = s +  e.JSPComment + "";
    
    return s;
  }
  else {
    for (var i in e) {
      
      if (e[i].comment.length > 1)
          com = '\n{' + e[i].comment.slice(2, this.length-3) + '}\n';
      if (e[i].JSPComment.length > 1)
          jspcom = e[i].JSPComment + '';
      if (e[i].LastComment.length > 1)
          lastcom = '{' + e[i].LastComment.slice(2, this.length-3) + '}';

      //if (e[i].JSPComment.length == 0 && e[i].comment.length > 0) { s =  s + "{" + e[i].comment.slice(2,e[i].comment.length-3) + "}\n"  ; }
      s= s+com;

      s = s + "\n{"+e[i].Row+"} " + e[i].Type + "(";
      s = s+e[i].Pos + ")";
      s = s + e[i].Opts + ";\n";
      s = s.replace('))[', ')[');
      //if (e[i].JSPComment.length > 0) s = s +  e[i].JSPComment + "\n";

      s = s+jspcom;
      s = s+lastcom;

      com = '';
      jspcom = '';
      lastcom = '';

    }
    if (withtags){
      return '<font size="4" style="letter-spacing:1.5px;"><b>JavaSketchpad code:</b></font><input onclick="copyJSPToClipboard()"" value="Copy JSP code to clipboard" type="button" class="browseButton"> <br><textarea id="jspgeneratedcode" cols="120" rows="50"><APPLET \nCODEBASE="jsp" ARCHIVE="jsp4.jar" CODE="GSP.class" WIDTH='+w+' HEIGHT='+h+' ALIGN=Left>\n<PARAM NAME=Offscreen VALUE=1><PARAM NAME=Frame VALUE=1>\n<PARAM NAME=MeasureInDegrees VALUE=1><PARAM NAME=DirectedAngles VALUE=0>\n<PARAM NAME=BackRed VALUE=250><PARAM NAME=BackGreen VALUE=250><PARAM NAME=BackBlue VALUE=250>\n<PARAM NAME=Construction VALUE="\n'+s+end+'">\n</APPLET></textarea>';
    }
    else
      return '<font size="4" style="letter-spacing:1.5px;"><b>JavaSketchpad code:</b></font><input onclick="copyJSPToClipboard()"" value="Copy JSP code to clipboard" type="button" class="browseButton"> <br><textarea id="jspgeneratedcode" cols="120" rows="50">' + s + end +'</textarea>';
  }

}

  function getJSPCmndFromString(s, i, c, m) {
    var goodChars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_/";
    var value = "";
    var ind = 0;
    if (i>=0) ind = i;
    else if (m) ind = s.indexOf(m, s.indexOf(c));
    else ind = s.indexOf(c) + c.length; 
    

    while(s.charAt(ind) == ' ' || goodChars.indexOf(s.charAt(ind))<0) ind = ind+1;

    while(goodChars.indexOf(s.charAt(ind))>=0) {
      value = value + s[ind];
      ind=ind+1;
    }
    return value;
  }

  /**
 * Function to change coordinates so that the (0,0) is the origo.
 * @method changeToCoord
 * @param {Int} x - x coordinate in Java Sketchpad form
 * @param {Int} y - y coordinate in Java Sketchpad form
 * @param {Array<Int>} appletsize - appletsize 
 * @param {Int} coord_div - coordinates dividor if coordinates has been set in Java Sketchpad code using Origin&Unit and Unitpoint commands
 * @param {Double} scalex - factor that is used in scaling the image horizontally
 * @param {Double} scaley - factor that is used in scaling the image vertically (this is not actually used, since the scalex and scaley are defines as the same value in main function)
 * @param {String} hasorigin - coordinates of origin if it has been changes in original Java Sketchapd code
 * @return {Array} - created coordinates (Array)
 */
function changeToCoord(x,y,appletsize, coord_div, scalex, scaley, hasorign) {
  var origin, moved_origin;
  origin = [appletsize[0]/coord_div/2, appletsize[1]/coord_div/2];
  if (hasorign.length > 0) {
    moved_origin = eval('['+hasorign+']');
    //moved_origin = hasorign;
    moved_origin[0] = moved_origin[0]*scalex; moved_origin[1] = moved_origin[1]*scaley; //alert(moved_origin);
    x = (-1*(origin[0]-((scalex*x)/coord_div))-((moved_origin[0]-(appletsize[0]/2))/coord_div));
    y = (origin[1]-((scaley*y)/coord_div)+((moved_origin[1]-(appletsize[1]/2))/coord_div));
  }
  else {
    x = (-1*(origin[0]-((scalex*x)/coord_div)));
    y = (origin[1]-((scaley*y)/coord_div));
  }
  return [Math.round(x*100)/100, Math.round(y*100)/100];
}

/**
 * Checks if there is Origin&Unit constructor in code, and return originelements rownr (String).
 * @method getOrigin
 * @param {String} a - original Java Sketchpad code
 * @return {String} org - information that is in the Origin&Unit line between the brackets '(' and ')'
 */
function getOrigin(a) {
  var rw="", ind, org;
  if (a.indexOf('Origin&Unit')>=0)
    rw = a.slice(a.indexOf('(', a.indexOf('Origin&Unit'))+1, a.indexOf(',', a.indexOf('Origin&Unit')));
  else if (a.indexOf('OriginAndUnit')>=0)
    rw = a.slice(a.indexOf('(', a.indexOf('OriginAndUnit'))+1, a.indexOf(',', a.indexOf('OriginAndUnit')));
  rw = '{'+rw+'}';
  org = a.slice(a.indexOf('(',a.indexOf(rw))+1, a.indexOf(')',a.indexOf(rw)));
  //alert(org);
  return org;
}