
/*
    JSP to JSXG 1.8.423

    2016-2018
      Henri Tanskanen

    This file is part of JSP to JSXG convertor.

    JSP to JSXG converter is free to use tool under UEF licence. 
    
    JSP to JSXG was programmed during working proccess of my
    Master's Thesis. Purpose was mainly to use it in converting
    previously made JSP sketches in new form. For myself this tool
    has also been in use for several times just to simplify the creation
    of a JSXGraph image, since the JSP syntax is easier to write.

    JSP to JSXG convertor is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the implied 
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

*/


//macros update 
var jsp_macros = "";



/** JAVASKETCHPAD TO JSXGRAPH CONVERTER : MAIN FUNCTION
 * This is the main function, that creates converted JSXGraph code from code-input and other input elements.
 * This function connects all the parts that are needed when constructing JSXG element from JSP element.
 * Input variables are:
 * code_input that is jsp code we want to convert (String)
 * size_input (optional) has table size information as width and height for example [400,400] (Array)
 * elementname_input has the element name information (String)
 * isFullPageConvert has the info if we are using fullpage convertor (boolean)
 * @param {String} code_input - original Java Sketchpad code
 * @param {Array<Int>} size_input - tablesize
 * @param {String} elementname_input - element name
 * @param {Boolean} isFullPageConvert - if is full page conver
 * @return {String} codestring - fully converted code to JSXGraph
 */

function jspToJsx(code_input, size_input, elementname_input, isFullPageConvert) {
  
  var appletsize = [400,400], 
    boardsize = [-5,5,5,-5],
    drawaxis = false,
    zoom = 0.9,  
    unitx = 20,
    unity = 20,
    coord_div=1,
    unit_in_pixels = 1,
    locusPoints=30,
    scalex = 1.0,
    scaley = 1.0,
    mesaccu = 10,
    animticks = 50,
    locus_counter = 0,
    borders_on = 0,
    butvertical = 0,
    startingtime = 0,
    endingtime = 0,
    executiontime = 0,
    directed_angles = 1,
    width = document.getElementById('boardwidthinput'),
    height = document.getElementById('boardheightinput'),
    curvelocus = document.getElementById('curve_locus').checked,
    evas = [],
    codestring = "",
    boardname = "",
    elid = "",
    teststring = "<h3><u>Developer attributes</u></h3>", 
    origstring = "<h3><u>JavaSketchpad code</u></h3>", 
    tmpstring = "",
    errors = "<u>Errors:</u><br><br>",
    sketch = "",
    jsxrenderer = "",
    has_origin_unit = "",
    boardstyle = "",
    boarddiv = "",
    board_update_events = "",
    scriptsource,
    mathjaxsource,
    backgroundcolour,
    back_red,
    back_green,
    back_blue,
    font_family,
    sgrid,
    use_hide_el = true,
    buttonstyle="<sty"+"le>.jxgbox button {padding:",
    animtype,
    gridlayer;
var sketchrows = [],
    sketchrows_comment =[],
    elr =[],
    el =[],
    calc = [],     
    animate = [],
    right_side_titles_and_buttons = [];
  
  // clear all html elements
try {  
  clearAll(['jsxgimage','jsxgcode','JSketchPadCode','testing', 'errormessages'], ['err_comms', 'err_info', 'err_submit_btn', 'ERRORS']);

//if (errors.length >25 )
  //errors = "<u>Errors:</u><br><br>";

  /* reading sketchcode and setting 
   * coordinate system (if in code command UnitPoint, else 100px)
   */
  if (code_input.length>12)
    sketch = code_input;
  else 
    sketch = getUserVal(code_input).replace(/(\r\n|\n|\r|" ")/gm,"");
  sketch = sketch.replace(/(\t)/gm, " ");
  sketch = sketch.trim();

  //BREAK IF CODE WONT PASS DUMMYTEST 
  if (!codeDummyTest(sketch)) {
    alert('There is some problems in your JSP-code. Convertion was stopped. Check your code!');
    return false;
    //
  }

  //input codearea to err_jspcode for submitting errors
  document.getElementById('err_jspcode').value = sketch;

  //check codestyle (and switch from JSP-$ if needed)
  if(!checkJSPStyle(sketch)) sketch = varJSPtoNumJSP(sketch);

    /* scaling factors
   */
  scalex = getUserVal('scalewidth');
  scaley = scalex; //document.getElementById('scaleheight').value;

 /* setting coordinate system
 */
  if (contains(sketch,"UnitPoint")) {
    coord_div=eval(sketch.slice(sketch.indexOf(",", sketch.indexOf("UnitPoint"))+1, sketch.indexOf(")", sketch.indexOf("UnitPoint")))) * scalex;
    unit_in_pixels = eval(coord_div);
  }

  //background colour option added 6.4.2018
  /*if (sketch.indexOf('BackRed')>=0 && sketch.indexOf('BackBlue')>=0 && sketch.indexOf('BackGreen')>=0) {
    back_red = eval(sketch.slice(sketch.indexOf('=', sketch.indexOf('BackRed'))+1, ) )
  }*/


  if (contains(sketch,"Origin&Unit") || contains(sketch,"OriginAndUnit")) {
    if (sketch.indexOf("$")<0) {
      has_origin_unit = getOrigin(sketch);
    }
  }

  //angles
  if (contains(sketch,'DirectedAngles')) {
    try {
      if (sketch[sketch.indexOf('=', sketch.indexOf('DirectedAngles'))+1] != '"')
        directed_angles = eval(sketch.slice(sketch.indexOf('=', sketch.indexOf('DirectedAngles'))+1, sketch.indexOf('=', sketch.indexOf('DirectedAngles'))+2));
      else if (sketch[sketch.indexOf('=', sketch.indexOf('DirectedAngles'))+1] == '"')
        directed_angles = eval(sketch.slice(sketch.indexOf('=', sketch.indexOf('DirectedAngles'))+2, sketch.indexOf('=', sketch.indexOf('DirectedAngles'))+3));
      else {alert('Problem in reading param DirectedAngle, this was set 0'); directed_angles=0}
    } catch(err) {errors = errors +"Convertor error, <br> JSP Param 'DirectedAngles' caused this error. <br>"+ err.message;}
  }

  //board colour attributes
  if (contains(sketch,'BackRed')) {try{back_red = eval(getValueFromString(sketch, 'BackRed', '=')); }catch(e){alert(e.message)}}
  if (contains(sketch,'BackGreen')) {try {back_green = eval(getValueFromString(sketch, 'BackGreen', '=')); }catch(e){alert(e.message)}}
  if (contains(sketch,'BackBlue')) {try {back_blue = eval(getValueFromString(sketch, 'BackBlue', '=')); }catch(e){alert(e.message)}}
  if (back_red && back_green && back_blue) backgroundcolour = rgbToHex(back_red, back_green, back_blue);
  else backgroundcolour = "FFFFFF";
  if (getUserVal('forcewhiteback')) backgroundcolour = "FFFFFF";
 
 //board font-family attribute
  //if (sketch.indexOf('TextFont')>=0){try{font_family = getValueFromString(sketch, 'TextFont'); } catch(e) {alert(e.message)} }
  
  /* appletsize
   * from input texy field or as user input from code
   * */
  var size_values_in_code = false;

  if ((contains(sketch,"WIDTH") || contains(sketch,"Width") || contains(sketch,"width")) && (contains(sketch,"HEIGHT") || contains(sketch,"Height") || contains(sketch,"height"))) size_values_in_code = true;

  if (size_input && size_input != false) {
    appletsize = size_input;
    width.value = size_input[0] * scalex;
    height.value = size_input[1] * scaley;
  }
  else {
    if ((!width.value || !height.value) && !size_values_in_code) errors = errors + "Width and/or height value is missing!<br>";
    appletsize = [width.value * scalex, height.value * scaley];
    if (size_values_in_code) {

      //appletsize = [sketch.slice(sketch.indexOf("WIDTH=")+6, sketch.indexOf("HEIGHT=")) * scalex, sketch.slice(sketch.indexOf("HEIGHT=")+7, sketch.indexOf("HEIGHT=")+10) * scaley];
      //width.value = sketch.slice(sketch.indexOf("WIDTH=")+6, sketch.indexOf("HEIGHT="));
      //height.value = sketch.slice(sketch.indexOf("HEIGHT=")+7, sketch.indexOf("HEIGHT=")+10);
      
      width.value = getBoardSizeFromCode(sketch)[0];
      height.value = getBoardSizeFromCode(sketch)[1];
      appletsize = [width.value * scalex, height.value * scaley];
    }
  }
  /* boardsize
   * initializing by appletsize
   */
  boardsize = [(appletsize[0]/coord_div/2)*-1, appletsize[1]/coord_div/2, appletsize[0]/coord_div/2, (appletsize[1]/coord_div/2)*-1]
  for (var i=0; i<boardsize.length; i++) {
    boardsize[i] = Math.round(boardsize[i]*100)/100;
  }
  /* setting unitx and unity
   * for hiding buttons outside the borders of the image
   */
  unitx = scalex*appletsize[0]/coord_div,
  unity = scaley*appletsize[1]/coord_div;

  /* getting user option of draw axis (yes/no)
   */
  drawaxis = getUserVal('drawaxis');
  /* reading user input for zoom option (default value is 0.9)
   */
  zoom = getUserVal('zoom');

  /* reading user input for measurement accuracy option (default value is one decimal)
   */
  mesaccu = getUserVal('measurementaccuracy');

  /* show grid (yes/no)
   */
  sgrid = getUserVal('showgrid');

  /* added grid layers
   */
  if (document.getElementById('addgridlayer'))
    gridlayer = eval(document.getElementById('addgridlayer').value.slice(document.getElementById('addgridlayer').value.length-1));
  else gridlayer = 0;
  // reading user input for boardname & element id name  
   //
  if (elementname_input) {
    boardname = elementname_input;
    elid = elementname_input;
  }
  if (!isFullPageConvert) {
    if ((sketch.indexOf('JSX board name')>=0 && sketch.indexOf('JSX element id')>=0) || sketch.indexOf('JSX boardname')>=0 && sketch.indexOf('JSX element id')>=0) {
      if (sketch.indexOf('JSX board name')>=0) boardname = getValueFromString(sketch, 'JSX board name');
      else boardname = getValueFromString(sketch, 'JSX boardname');
      elid = getValueFromString(sketch, 'JSX element id');
      document.getElementById('jsxgboardname').value = boardname;
      document.getElementById('jsxgelementid').value = elid;
    }
    else {
      boardname = document.getElementById('jsxgboardname').value;
      elid = document.getElementById('jsxgelementid').value;
      if (boardname == '' && elid == '') {boardname = 'JXGboard'; elid = 'el'}
      else if (boardname == '' ) boardname = 'JXGboard';
      else if (elid == '' ) elid = 'el';
    }
  }
  if (boardname.indexOf(' ')>=0 || elid.indexOf(' ')>=0)
    alert("Spaces not allowed in JSXGraph board name or element id!");


  // reading user input for jsxgraphcore.js source
  scriptsource = document.getElementById('source').value;

  //jsx renderer
  if (document.getElementById('jsxrenderer_sel').value != 'default') 
    jsxrenderer = "\trenderer:'"+ document.getElementById('jsxrenderer_sel').value +"',\n ";

  // borders on?
  if (document.getElementById('jsxgimageborders').checked == true) borders_on = 1; 

  //hiding option 12.3.2018
  if (document.getElementById('hide_element').checked == true) use_hide_el = false;

  // getting animation type option
  animtype = document.getElementById('animationsel').value;
  if (animtype != "animtype1") animticks = eval(animtype.slice(10, 12));

  //buttonstyle
  if (document.getElementById('usebuttonstyle').checked == true) {
      buttonstyle = "#"+boardname+
                  " button {padding:" + document.getElementById('buttonpaddingsel').value + 
                  "margin-top:"+ document.getElementById('buttonmargintopsel').value +
                  "background-color:"+ document.getElementById('buttonbackgroundcolorsel').value +
                  "}\n";
  } else buttonstyle = "";

  boardstyle = "<sty"+"le>#"+boardname + " {\n" +
              "\tborder-style:solid;\n"+
              "\tborder-width:"+borders_on+"px;\n"+
              "\twidth:"+appletsize[0]+"px;\n"+
              "\theight: "+appletsize[1]+"px;\n"+
              "\toverflow: hidden;\n"+
              "\tposition: relative;\n"+
              "\tbackground-color:#"+backgroundcolour+";"+
              "}\n"+
              "\t"+buttonstyle+
              "</s"+"tyle>\n";
  boarddiv = "<div class='jxgbox' id='"+boardname+"' > </div>"


  // initializing board element for the html page (this program)

    // korvattu 7.3.2018 paremmalla ratkaisulla // document.getElementById("jsxgimage").innerHTML = buttonstyle.replace('#'+boardname, '.jxgbox')+"<h3 style='margin:2px 0px 0px 5px; text-decoration:none; letter-spacing:1.5px;'>JSXGraph image</h3><div class='jxgbox' id='JSXBoard' style='border-style:solid; border-width:"+borders_on+"px; width:"+appletsize[0]+"px; height: "+appletsize[1]+"px; overflow: hidden; position: relative; background-color:white;'> </div>";
    
    document.getElementById("jsxgimage").innerHTML = /*buttonstyle.replace('#'+boardname, '.jxgbox')+*/boardstyle+boarddiv;
 
  // writing var board iniatilizing and variable definitions into codestring, evaluating them
   // and replacing boardname for generated code   
  codestring = "var "+boardname+
               "_b = JXG.JSXGraph.initBoard('"+boardname+"', {\n"+
               jsxrenderer+
               "\tboundingbox:["+boardsize+"],\n"+
               "\toriginX:"+Math.round(100*(scalex*appletsize[0]/coord_div/2))/100+", originY:"+Math.round(100*(scaley*appletsize[1]/coord_div/2))/100+",\n"+
               "\tzoomX:"+zoom+", zoomY:"+zoom+",\n"+
               "\tkeepaspectratio:false,\n"+
               "\taxis:"+drawaxis+",\n"+
               "\tshowCopyright: false,\n"+
               "\tshowNavigation: false,\n"+
               "\tgrid: "+sgrid+",\n";
               if (document.getElementById('cleartracebutton').value == 'show' || (document.getElementById('cleartracebutton').value == 'auto' && sketch.indexOf('traced')>0))
                codestring = codestring + "\tshowClearTraces: true\n";
               codestring = codestring + 
               "} );\n"+
              // defining variables that will be written in converter code
              "var "+
              elid+"_el=[],\n"+
              elid+"_tel=[],\n"+
              elid+"_transf=[],\n"+
              elid+"_transf2=[],\n"+
              elid+"_animOn=false;\n";

  if (gridlayer > 0) {
    for (var i=0; i<gridlayer; i++) {
      codestring = codestring + boardname + "_b.create('grid', []);\n";
    }
  }

  // JXG.Options.text.useMathJax = true;
  
 
    eval(codestring); // eavaluating board and varaibles for the sample sketch in the right column of converter page
  
  if (!isFullPageConvert)
    codestring = codestring + "\n";
    //codestring = codestring + "&#13;&#10;&#13;&#10;";
  codestring = codestring.replace("JSXBoard",boardname); // replacing boardname with user defined name

  sketchrows = separateCommands(sketch); // getting the rows from the JSP code
  
  for (var i=0; i<sketchrows.length; i++) {
    elr[i] = sketchpadrowToJsxgElement(sketchrows[i]); // each row into JSXG element
    //alert(elr[i].Row);
    //origstring = origstring + sketchrows[i] +"<br>"; 
  }
  //alert(elr[elr.length-1].lastComment);

  if ((document.getElementById('originalcode').value == 'Yes' || document.getElementById('originalcode').value == 'Yes with tags') && !isFullPageConvert) {
    if (document.getElementById('originalcode').value == 'Yes with tags')
      document.getElementById("JSketchPadCode").innerHTML = getOrigString(elr, width.value, height.value, true);
    else
      document.getElementById("JSketchPadCode").innerHTML = getOrigString(elr, width.value, height.value, false);
  }

  // Change attributes 
  for (var i=0; i<elr.length; i++) {

      changeTypeToJSX(elr[i]);
      //alert(elr[i].Row + elr[i].Type);
      changePosInfoToJSX(elr[i],appletsize,coord_div,scalex,scaley,has_origin_unit);
      changeAttributesToJSX(elr[i]);
      changeOptionsToJSX(elr[i]);
      // create teststring for developer usage, prints all the object values
      teststring = teststring + "Row: "+elr[i].Row +", Type: "+elr[i].Type+", JsxType: "+elr[i].JsxType+", Pos: "+elr[i].Pos+", Opt: "+elr[i].Opts+", Att.Info: "+elr[i].Att.Info+ ", Anim: "+elr[i].Anim+"<br>";
    
    
  }
  if (document.getElementById('developerstuff').checked == true && !isFullPageConvert)
    document.getElementById("testing").innerHTML = teststring;
  
  for (var i=0; i<elr.length;i++) {

      el[elr[i].Row] = elr[i];
      sketchrows_comment[elr[i].Row] = elr[i];

  }

  //MathJax in code
  var add_mathjax = "";
  if (document.getElementById('addmathjax').checked == true)
    add_mathjax = "<"+"scr"+"ipt src='" + document.getElementById('mathjax_source').value + "'>"+"<"+"/"+"scr"+"ipt>\n";
  else add_mathjax = "";
  

  // Beginning of the codestring for the user  
  codestring = codestring+ 
               "<><script src ='" + scriptsource +"'> <"+"/scr"+"ipt"+"> \n"+ 
               add_mathjax + 
               boardstyle + 
               boarddiv +"\n<"+
               "script"+">//![CDATA[\n"+
               "(function(){\n"+
               codestring;
 
  //this is to calculate time that convertion takes
  var startingtime = new Date().getTime();
} catch(e) {alert(e.message);}


  //for (var i=0; i<el.length; i++) {
    try {
  for (var i in el) {

    if (el[i].Att.Stop)
      break;


    else if (el[i].Row ) {

      var obj_c = elid+"_el["+el[i].Row+"] = "+boardname+"_b.create";


      



      // POINTS * * * * /
      if (el[i].JsxType == 'point') {
        evas[el[i].Row] = obj_c+"('point', ["+el[i].Pos+"],"+setAttributes(el[i],'',use_hide_el,elid)+");";
      }
      else if (el[i].JsxType == 'unitpoint'){
        evas[el[i].Row] = obj_c+"('point', [function(){return "+elid+"_el["+el[i].Pos[0]+"].X()+"+1+";}, function(){return "+elid+"_el["+el[i].Pos[0]+"].Y();}],"+setAttributes(el[i],'',use_hide_el,elid)+" );";
      }
      else if (el[i].JsxType == 'midpoint'){
        evas[el[i].Row] = obj_c+"('midpoint', ["+elid+"_el["+el[i].Pos[0]+"]],"+setAttributes(el[i],'',use_hide_el,elid)+");";
      }
      else if (el[i].JsxType == 'intersection') {
        evas[el[i].Row] = obj_c+"('intersection', ["+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"], "+el[i].Att.Info+"],"+setAttributes(el[i],'alwaysIntersect:false',use_hide_el,elid)+");";
      }
      else if (el[i].JsxType == 'glider') {
        tmpstring = gliderPoint(eval(elid+"_el["+el[i].Pos[0]+"]"),el[i].Pos[1],eval(elid+"_el["+el[el[i].Pos[0]].Pos[0]+"]"))[0]+","+gliderPoint(eval(elid+"_el["+el[i].Pos[0]+"]"),el[i].Pos[1],eval(elid+"_el["+el[el[i].Pos[0]].Pos[0]+"]"))[1];
        evas[el[i].Row] = obj_c+"('glider', ["+tmpstring+","+elid+"_el["+el[i].Pos[0]+"]],"+setAttributes(el[i],'',use_hide_el,elid)+");";
      }
      else if (el[i].JsxType == 'plotxy') {
        // PlotXY(objRef1, objRef2, objRef3);
         // Defines the point on the coordinate system objRef2 specified horizontally by the value of measurement objRef3 and vertically by the value of objRef1. 
         // That is, PlotXY() plots the point (objRef3, objRef1) on objRef2.
         //
        evas[el[i].Row] = obj_c+"('point', [function() {return "+elid+"_el["+el[el[i].Pos[1]].Pos[0]+"].X() + "+calc[el[i].Pos[2]]+";}, function() {return "+elid+"_el["+el[el[i].Pos[1]].Pos[0]+"].Y() + "+calc[el[i].Pos[0]]+";}],"+setAttributes(el[i],'',use_hide_el,elid)+");";
      }

      // * * * * /POINTS /





      // CIRCLES * * * * * /

      else if (el[i].JsxType == 'circle') {
        evas[el[i].Row] = obj_c+"('"+el[i].JsxType+"', ["+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"]],"+setAttributes(el[i],'',use_hide_el,elid)+"); ";
      }
      else if (el[i].JsxType == 'circle by radius') {
        evas[el[i].Row] = obj_c+"('circle', ["+elid+"_el["+el[i].Pos[0]+"], function(){return "+elid+"_el["+el[i].Pos[1]+"].L();}],"+setAttributes(el[i],'',use_hide_el,elid)+"); ";
      }

      // * * * * * * /CIRCLES /




      // STAIGHT LINES * * * * /

      else if (el[i].JsxType == 'line' || el[i].JsxType == 'perpendicular' || el[i].JsxType == 'parallel') {
        evas[el[i].Row] = obj_c+"('"+el[i].JsxType+"', ["+elid+"_el["+el[i].Pos[1]+"],"+elid+"_el["+el[i].Pos[0]+"]],"+setAttributes(el[i],'',use_hide_el,elid)+");";
      }
      else if (el[i].JsxType == 'segment') {
        evas[el[i].Row] = obj_c+"('"+el[i].JsxType+"', ["+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"]],"+setAttributes(el[i],'',use_hide_el,elid)+");";
      }
      else if (el[i].JsxType == 'ray') {
        evas[el[i].Row] = obj_c+"('line', ["+elid+"_el["+el[i].Pos[1]+"],"+elid+"_el["+el[i].Pos[0]+"]],"+setAttributes(el[i],'',use_hide_el,elid)+");";    
      }
      else if (el[i].JsxType == 'axisx') {
          evas[el[i].Row] = obj_c+"('axis', [["+elid+"_el["+el[i].Pos[0]+"][0].X(),"+elid+"_el["+el[i].Pos[0]+"][0].Y()], ["+elid+"_el["+el[i].Pos[0]+"][0].X()+1, "+elid+"_el["+el[i].Pos[0]+"][0].Y()]],"+setAttributes(el[i], '')+"); " +elid+"_el["+el[i].Row+"].removeAllTicks();";       
      }
      else if (el[i].JsxType == 'axisy') {
          evas[el[i].Row] = obj_c+"('axis', [["+elid+"_el["+el[i].Pos[0]+"][0].X(),"+elid+"_el["+el[i].Pos[0]+"][0].Y()], ["+elid+"_el["+el[i].Pos[0]+"][0].X(), "+elid+"_el["+el[i].Pos[0]+"][0].Y()+1]],"+setAttributes(el[i], '')+"); "+elid+"_el["+el[i].Row+"].removeAllTicks();";        
      }
      // * * * * /STRAIGHT LINES /





      // INTERIORS * * * * * /

      else if (el[i].JsxType == 'polygon') {
        for (var j=0; j<el[i].Pos.length; j++) {
          if (j != el[i].Pos.length-1) tmpstring = tmpstring + elid+"_el["+el[i].Pos[j]+"],"; else tmpstring = tmpstring + elid+"_el["+el[i].Pos[j]+"],"; }
            evas[el[i].Row] = obj_c+"('polygon', ["+tmpstring+"],"+setAttributes(el[i],'',use_hide_el,elid)+");";
      } 
      else if (el[i].JsxType == 'circle interior') {
        if ((el[el[i].Pos[0]].JsxType == 'circle' || el[el[i].Pos[0]].JsxType == 'circle by radius') && el[i].Opts.indexOf('visible:false')<0) {
          evas[el[i].Row] = elid+"_el["+el[i].Pos[0]+"].setAttribute({color:'"+el[i].Att.Color+"'});"; }
        else evas[el[i].Row] = ""; 
      } 

      // * * * * /INTERIORS /





      // TRANSFORMATIONS * * * * /

      else if (el[i].JsxType == 'reflection') {
        evas[el[i].Row] = obj_c+"('"+el[i].JsxType+"', ["+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"]],"+setAttributes(el[i],'',use_hide_el,elid)+");";
      }
      else if (el[i].JsxType == 'dilation') {
        evas[el[i].Row] = obj_c+"('point', [function(){return "+el[i].Pos[2]+"*("+elid+"_el["+el[i].Pos[0]+"].X()-"+elid+"_el["+el[i].Pos[1]+"].X())+"+elid+"_el["+el[i].Pos[1]+"].X();},function(){return "+el[i].Pos[2]+"*("+elid+"_el["+el[i].Pos[0]+"].Y()-"+elid+"_el["+el[i].Pos[1]+"].Y())+"+elid+"_el["+el[i].Pos[1]+"].Y();}],"+setAttributes(el[i],'',use_hide_el,elid)+");";
      }
      else if (el[i].JsxType == 'dilation/segmentratio') {
        evas[el[i].Row] = obj_c+"('point', [function(){return ("+elid+"_el["+el[i].Pos[2]+"].L()/"+elid+"_el["+el[i].Pos[3]+"].L())*("+elid+"_el["+el[i].Pos[0]+"].X()-"+elid+"_el["+el[i].Pos[1]+"].X())+"+elid+"_el["+el[i].Pos[1]+"].X();},function(){return ("+elid+"_el["+el[i].Pos[2]+"].L()/"+elid+"_el["+el[i].Pos[3]+"].L())*("+elid+"_el["+el[i].Pos[0]+"].Y()-"+elid+"_el["+el[i].Pos[1]+"].Y())+"+elid+"_el["+el[i].Pos[1]+"].Y();}],"+setAttributes(el[i],'',use_hide_el,elid)+");";
      }
      else if (el[i].JsxType == 'dilation/3ptratio') {
        // Dilation/3PtRatio (objRef1, objRef2, objRef3, objRef4, objRef5);
         // Defines the image of any objRef1 dilated with respect to center point objRef2 by the signed ratio implied by the distances between points objRef3, 4, and 5. 
         // If these three points are A, B, and C, respectively, the three-point ratio implied by them is |AC|/|AB|. 
        evas[el[i].Row] = obj_c+"('point', [function(){return Math.sign(Math.cos(JXG.Math.Geometry.rad("+elid+"_el["+el[i].Pos[3]+"], "+elid+"_el["+el[i].Pos[2]+"], "+elid+"_el["+el[i].Pos[4]+"]))) * ("+elid+"_el["+el[i].Pos[2]+"].Dist("+elid+"_el["+el[i].Pos[4]+"]) / "+elid+"_el["+el[i].Pos[2]+"].Dist("+elid+"_el["+el[i].Pos[3]+"])) * ("+elid+"_el["+el[i].Pos[0]+"].X() - "+elid+"_el["+el[i].Pos[1]+"].X()) + "+elid+"_el["+el[i].Pos[1]+"].X()},function() {return Math.sign(Math.cos(JXG.Math.Geometry.rad("+elid+"_el["+el[i].Pos[3]+"], "+elid+"_el["+el[i].Pos[2]+"], "+elid+"_el["+el[i].Pos[4]+"]))) * ("+elid+"_el["+el[i].Pos[2]+"].Dist("+elid+"_el["+el[i].Pos[4]+"]) / "+elid+"_el["+el[i].Pos[2]+"].Dist("+elid+"_el["+el[i].Pos[3]+"])) * ("+elid+"_el["+el[i].Pos[0]+"].Y() - "+elid+"_el["+el[i].Pos[1]+"].Y()) + "+elid+"_el["+el[i].Pos[1]+"].Y()}],"+setAttributes(el[i],'',use_hide_el,elid)+");";

      }
      else if (el[i].JsxType == 'dilation/markedratio') {
        evas[el[i].Row] = obj_c+"('point', [function(){var ratio = "+calc[el[i].Pos[2]]+";return ratio*("+elid+"_el["+el[i].Pos[0]+"].X()-"+elid+"_el["+el[i].Pos[1]+"].X())+"+elid+"_el["+el[i].Pos[1]+"].X();},function(){var ratio = "+calc[el[i].Pos[2]]+";return ratio*("+elid+"_el["+el[i].Pos[0]+"].Y()-"+elid+"_el["+el[i].Pos[1]+"].Y())+"+elid+"_el["+el[i].Pos[1]+"].Y();}],"+setAttributes(el[i],'',use_hide_el,elid)+");";
      }
      else if (el[i].JsxType == 'rotation') {
      //5.5.6 Rotation (objRef1, objRef2, num);
        //Defines the image of any objRef1 rotated around center point objRef2 by the radian angle specified by num.
        //eval(elid+"_transf[el[i].Row] = "+boardname+"_b.create('transform', ["+el[i].Pos[2]+","+elid+"_el[el[i].Pos[1]]], {type:'rotate'})");          
        //codestring = codestring + 
        evas[el[i].Row] = elid+"_transf["+el[i].Row+"] = "+boardname+"_b.create('transform', ["+el[i].Pos[2]+","+elid+"_el["+el[i].Pos[1]+"]], {type:'rotate'});";
        evas[el[i].Row] = evas[el[i].Row] + obj_c+"('point', ["+elid+"_el["+el[i].Pos[0]+"], "+elid+"_transf["+el[i].Row+"]], {name:'"+el[i].Att.Name+"', color:'"+el[i].Att.Color+"', visible:"+el[i].Att.Visible+", fixed:"+elid+"_el["+el[i].Pos[0]+"].getAttribute, size:"+el[i].Att.Size+", showInfoBox:"+el[i].Att.Infobox+", label:"+el[i].Att.LabelAlign+"} );";
      }     
      else if (el[i].JsxType == 'rotation/measuredangle') {
        //5.5.6 Rotation (objRef1, objRef2, objRef3);
         // Defines the image of any objRef1 rotated around center point objRef2 by the radian angle expressed by measurement objRef3.   
        if (eval(elid+"_el["+el[i].Pos[0]+"].getType()")=='circle') {
          //el[i].CType = "circles";
          //evas[el[i].Row] = elid+"_transf["+el[i].Row+"] = "+boardname+"_b.create('transform', [function(){return "+calc[+el[i].Pos[2]]+"},"+elid+"_el["+el[i].Pos[1]+"]], {type:'rotate'});" + obj_c+"('point',["+elid+"_el["+el[i].Pos[0]+"].center, "+elid+"_transf["+el[i].Row+"]],"+setAttributes(el[i], "fixed:"+elid+"_el["+el[i].Pos[0]+"].getAttribute",use_hide_el)+"); "+boardname+"_b.create('circle', ["+elid+"_el["+el[i].Row+"], function(){return "+elid+"_el["+el[i].Pos[0]+"].radius;}],"+setAttributes(el[el[i].Pos[0]],'',use_hide_el,elid)+");";
          evas[el[i].Row] = elid+"_transf["+el[i].Row+"] = "+boardname+"_b.create('transform', [function(){return "+calc[+el[i].Pos[2]]+"},"+elid+"_el["+el[i].Pos[1]+"]], {type:'rotate'});" + elid+"_el["+el[i].Row+"] = " + boardname + "_b.create('point',["+elid+"_el["+el[i].Pos[0]+"].center, "+elid+"_transf["+el[i].Row+"]],"+setAttributes(el[i], "fixed:"+elid+"_el["+el[i].Pos[0]+"].getAttribute",use_hide_el)+"); "+elid+"_tel["+el[i].Row +"] = " + boardname + "_b.create('circle', ["+elid+"_el["+el[i].Row+"], function(){return "+elid+"_el["+el[i].Pos[0]+"].radius;}],"+setAttributes(el[/*el[i].Pos[0]*/el[i].Row],'',use_hide_el,elid,'circles')+");";
        } 
        else {
          evas[el[i].Row] = elid+"_transf["+el[i].Row+"] = "+boardname+"_b.create('transform', [function(){return "+calc[+el[i].Pos[2]]+"},"+elid+"_el["+el[i].Pos[1]+"]], {type:'rotate'});" + obj_c+"('point', ["+elid+"_el["+el[i].Pos[0]+"], "+elid+"_transf["+el[i].Row+"]],"+setAttributes(el[i], "fixed:"+elid+"_el["+el[i].Pos[0]+"].getAttribute", use_hide_el,elid)+");";
        }
        if(el[i].Opts.indexOf('traced')>=0 && el[i].Opts.indexOf('visible:false')<0) {
          //evas[el[i].Row] = evas[el[i].Row] + "\n\n"+ boardname+"_b.on('update', function(){if("+calc[+el[i].Pos[2]]+">=0 || "+calc[+el[i].Pos[2]]+"<0) "+elid+"_el["+el[i].Row+1 +"].setAttribute('trace:true'); });"
          board_update_events = board_update_events + "if("+calc[+el[i].Pos[2]]+">=0 || "+calc[+el[i].Pos[2]]+"<0) "+elid+"_tel["+el[i].Row +"].setAttribute('trace:true');";
        }
      } 
      else if (el[i].JsxType == 'rotation/markedangle') {
        /* Rotation/MarkedAngle (objRef1, objRef2, objRef3, objRef4, objRef5);
         * Defines the image of any objRef1 rotated around center point objRef2 by the directed angle implied by the three points objRef3-objRef4-objRef5.       
         * JXG.Math.Geometry.rad("+elid+"_el["+el[i].Pos[2]+"], "+elid+"_el["+el[i].Pos[3]+"], "+elid+"_el["+el[i].Pos[4]+"])
         */
        if (el[el[i].Pos[0]].CType == 'points'){  
          evas[el[i].Row] = elid+"_transf[el[i].Row] = "+boardname+"_b.create('transform', [function() {return JXG.Math.Geometry.rad("+elid+"_el["+el[i].Pos[2]+"], "+elid+"_el["+el[i].Pos[3]+"], "+elid+"_el["+el[i].Pos[4]+"]);},"+elid+"_el["+el[i].Pos[1]+"]], {type:'rotate'});\n" + obj_c+"('point', ["+elid+"_el["+el[i].Pos[0]+"], "+elid+"_transf["+el[i].Row+"]],"+setAttributes(el[i], "fixed:"+elid+"_el["+el[i].Pos[0]+"].getAttribute", use_hide_el,elid)+");";
          el[i].ParentType = 'points';
        } else if (el[el[i].Pos[0]].CType == 'straightlines') {
          evas[el[i].Row] = elid+"_transf["+el[i].Row+"] = "+boardname+"_b.create('transform', [function() {return JXG.Math.Geometry.rad("+elid+"_el["+el[i].Pos[2]+"], "+elid+"_el["+el[i].Pos[3]+"], "+elid+"_el["+el[i].Pos[4]+"]);},"+elid+"_el["+el[i].Pos[1]+"]], {type:'rotate'});"
          evas[el[i].Row] = evas[el[i].Row] + "\n" + obj_c + 
                            "('" + eval(elid+"_el["+el[i].Pos[0]+"]").getType() + "', ["+
                              "["+elid+"_el["+el[i].Pos[0]+"].point1, "+elid+"_transf["+el[i].Row+"]], "+
                              "["+elid+"_el["+el[i].Pos[0]+"].point2, "+elid+"_transf["+el[i].Row+"]]],"+setAttributes(el[el[i].Pos[0]], "fixed:"+elid+"_el["+el[i].Pos[0]+"].getAttribute", el[i] )+");";
          el[i].ParentType = 'straightlines';
        }
        else {
          alert('Rotation/MarkedAngle works only for points and straight lines for now... Convertion stopped on line ' + el[i].Row + "," + el[i].Type + "(" + el[i].Pos + ")[" + el[i].Opts.slice(1) + "];");
          return null;
        }

      } 
      else if (el[i].JsxType == 'translation') { 
        //eval(elid+"_transf[el[i].Row] = "+boardname+"_b.create('transform', ["+scalex * el[i].Pos[1]/coord_div +", "+scaley * -1 * el[i].Pos[2]/coord_div +"], {type:'translate'});" );
        //codestring = codestring + elid+"_transf["+el[i].Row+"] = "+boardname+"_b.create('transform', ["+scalex*el[i].Pos[1]/coord_div+","+ scaley*el[i].Pos[2]/coord_div+"], {type:'translate'});";
        evas[el[i].Row] = elid+"_transf["+el[i].Row+"] = "+boardname+"_b.create('transform', ["+scalex*el[i].Pos[1]/coord_div+","+ scaley*el[i].Pos[2]/coord_div+"], {type:'translate'});" + obj_c+"('point', ["+elid+"_el["+el[i].Pos[0]+"], "+elid+"_transf["+el[i].Row+"]],"+setAttributes(el[i], "fixed:"+elid+"_el["+el[i].Pos[0]+"].getAttribute", use_hide_el,elid)+");";
      }
      else if (el[i].JsxType == 'translation/fixedangle/markeddistance') {
        /* Translation/FixedAngle/MarkedDistance (objRef1, objRef2, num);
         * Defines the image of any objRef1 translated by the distance expressed by measurement objRef2 in the direction expressed by num radians.
         * [ https://www.math.uni-bielefeld.de/~lisken/jsp/qref.html ] 
         */
        if (el[el[i].Pos[0]].CType == 'points' || el[el[i].Pos[0]].CType == 'transformations' || jsp_macros.indexOf(el[el[i].Pos[0]].Type)>=0) { 
          eval(elid+"_transf[el[i].Row] = "+boardname+"_b.create('transform', [function(){return "+calc[el[i].Pos[1]]+";}, 0], {type:'translate'})");
          eval(elid+"_transf2[el[i].Row] = "+boardname+"_b.create('transform', ["+el[i].Pos[2]+", "+elid+"_el["+el[i].Pos[0]+"]], {type:'rotate'})");
          codestring = codestring + elid+"_transf["+el[i].Row+"] = "+boardname+"_b.create('transform', [function(){return "+calc[el[i].Pos[1]]+";}, 0], {type:'translate'});";
          codestring = codestring + elid+"_transf2["+el[i].Row+"] = "+boardname+"_b.create('transform', ["+el[i].Pos[2]+", "+elid+"_el["+el[i].Pos[0]+"]], {type:'rotate'});";
          evas[el[i].Row] = obj_c+"('point', ["+elid+"_el["+el[i].Pos[0]+"], ["+elid+"_transf["+el[i].Row+"],"+elid+"_transf2["+el[i].Row+"]]],"+setAttributes(el[i], "fixed:"+elid+"_el["+el[i].Pos[0]+"].getAttribute", use_hide_el,elid)+");";          
        }
        else if (el[el[i].Pos[0]].CType == 'straightlines') {
          eval(elid+"_transf[el[i].Row] = "+boardname+"_b.create('transform', [function(){return "+calc[el[i].Pos[1]]+";}, 0], {type:'translate'})");
          eval(elid+"_transf2[el[i].Row] = "+boardname+"_b.create('transform', ["+el[i].Pos[2]+", "+elid+"_el["+el[i].Pos[0]+"].point1], {type:'rotate'})");
          eval(elid+"_transf2[el[i].Row +1 ] = "+boardname+"_b.create('transform', ["+el[i].Pos[2]+", "+elid+"_el["+el[i].Pos[0]+"].point2], {type:'rotate'})");
          codestring = codestring + elid+"_transf["+el[i].Row+"] = "+boardname+"_b.create('transform', [function(){return "+calc[el[i].Pos[1]]+";}, 0], {type:'translate'});";
          codestring = codestring + elid+"_transf2["+el[i].Row+"] = "+boardname+"_b.create('transform', ["+el[i].Pos[2]+", "+elid+"_el["+el[i].Pos[0]+"].point1], {type:'rotate'});";
          codestring = codestring + elid+"_transf2["+el[i].Row + 1+"] = "+boardname+"_b.create('transform', ["+el[i].Pos[2]+", "+elid+"_el["+el[i].Pos[0]+"].point2], {type:'rotate'});";
          evas[el[i].Row] = obj_c+"('line', [["+elid+"_el["+el[i].Pos[0]+"].point1, ["+elid+"_transf["+el[i].Row+"],"+elid+"_transf2["+el[i].Row+"]]], ["+elid+"_el["+el[i].Pos[0]+"].point2, ["+elid+"_transf["+el[i].Row+"],"+elid+"_transf2["+el[i].Row+"+1]]]],"+setAttributes(el[i], "fixed:"+elid+"_el["+el[i].Pos[0]+"].getAttribute", use_hide_el,elid)+");";          
        }
        else {
          alert( el[i].Type + " not working for " + el[el[i].Pos[0]].CType + ", \nline " + el[i].Row + " in JSP code. \nConverting was stopped!");
          return 'null';
        }
      }     
      else if (el[i].JsxType == 'translation/markedangle/markeddistance') {
        /* Translation/MarkedAngle/MarkedDistance (objRef1, objRef2, objRef3);
         * Defines the image of any objRef1 translated by the distance expressed by distance measurement objRef3 in the direction expressed by angle measurement objRef2.
         * [ https://www.math.uni-bielefeld.de/~lisken/jsp/qref.html ] 
         */
        eval(elid+"_transf[el[i].Row] = "+boardname+"_b.create('transform', [function(){return Math.abs("+elid+"_el["+el[i].Pos[0]+"].X()-("+calc[el[i].Pos[2]]+"/coord_div))+"+elid+"_el["+el[i].Pos[0]+"].X();}, 0], {type:'translate'})");
        eval(elid+"_transf2[el[i].Row] = "+boardname+"_b.create('transform', [function(){return "+calc[el[i].Pos[1]]+";}, "+elid+"_el["+el[i].Pos[0]+"]], {type:'rotate'})");
        codestring = codestring + elid+"_transf["+el[i].Row+"] = "+boardname+"_b.create('transform', [function(){return Math.abs("+elid+"_el["+el[i].Pos[0]+"].X()-("+calc[el[i].Pos[2]]+"/coord_div))+"+elid+"_el["+el[i].Pos[0]+"].X();}, 0], {type:'translate'});";
        codestring = codestring + elid+"_transf2["+el[i].Row+"] = "+boardname+"_b.create('transform', [function(){return "+calc[el[i].Pos[1]]+";}, "+elid+"_el["+el[i].Pos[0]+"]], {type:'rotate'});";
        evas[el[i].Row] = obj_c+"('point', ["+elid+"_el["+el[i].Pos[0]+"], ["+elid+"_transf["+el[i].Row+"],"+elid+"_transf2["+el[i].Row+"]]],"+setAttributes(el[i], "fixed:"+elid+"_el["+el[i].Pos[0]+"].getAttribute", use_hide_el,elid)+");";   
      }   
      else if (el[i].JsxType == 'vectortranslation') {
        // VectorTranslation (objRef1, objRef2, objRef3);
         // Defines the image of any objRef1 translated by the vector implied by two points objRef2 -> objRef3. 
         // objRef2 is the foot of the vector, and objRef3 is its tail. 
        if (eval(elid+"_el["+el[i].Pos[0]+"].getType()")=='point' || eval(elid+"_el["+el[i].Pos[0]+"].getType()")=='glider' || el[el[i].Pos[0]].JsxType=='intersection' || eval(elid+"_el["+el[i].Pos[0]+"].getType()")=='reflection' || eval(elid+"_el["+el[i].Pos[0]+"].getType()")=='midpoint') {  
          //evas[el[i].Row] = obj_c+"('point', [function(){return "+elid+"_el["+el[i].Pos[0]+"].X()+(Math.sign("+elid+"_el["+el[i].Pos[2]+"].X()-"+elid+"_el["+el[i].Pos[1]+"].X()))*Math.abs("+elid+"_el["+el[i].Pos[1]+"].X()-"+elid+"_el["+el[i].Pos[2]+"].X());}, function(){return "+elid+"_el["+el[i].Pos[0]+"].Y()+(Math.sign("+elid+"_el["+el[i].Pos[2]+"].Y()-"+elid+"_el["+el[i].Pos[1]+"].Y()))*Math.abs("+elid+"_el["+el[i].Pos[1]+"].Y()-"+elid+"_el["+el[i].Pos[2]+"].Y());}],"+setAttributes(el[i],'',use_hide_el,elid)+");";
          //evas[el[i].Row] = obj_c+"('point', [function(){return "+elid+"_el["+el[i].Pos[0]+"].X()+("+elid+"_el["+el[i].Pos[2]+"].X()-"+elid+"_el["+el[i].Pos[1]+"].X());}, function(){return "+elid+"_el["+el[i].Pos[0]+"].Y()+("+elid+"_el["+el[i].Pos[2]+"].Y()-"+elid+"_el["+el[i].Pos[1]+"].Y());}],"+setAttributes(el[i],'',use_hide_el,elid)+");";
          eval (elid + "_transf[" + el[i].Row + "] = " + boardname + "_b.create('transform', [function(){return "+elid+"_el["+el[i].Pos[2]+"].X()-"+elid+"_el["+el[i].Pos[1]+"].X();},function(){return "+elid+"_el["+el[i].Pos[2]+"].Y()-"+elid+"_el["+el[i].Pos[1]+"].Y();}], {type:'translate'});");
          codestring = codestring + elid + "_transf[" + el[i].Row + "] = " + boardname + "_b.create('transform', [function(){return "+elid+"_el["+el[i].Pos[2]+"].X()-"+elid+"_el["+el[i].Pos[1]+"].X();},function(){return "+elid+"_el["+el[i].Pos[2]+"].Y()-"+elid+"_el["+el[i].Pos[1]+"].Y();}], {type:'translate'});";
          evas[el[i].Row] = obj_c+"('point', ["+elid+"_el["+el[i].Pos[0]+"], "+ elid+"_transf["+el[i].Row+"]],"+setAttributes(el[i],'',use_hide_el,elid)+");";
        }
        // FOR SEGMENT
        else if (eval(elid+"_el["+el[i].Pos[0]+"].getType()")=='segment') {
          if (el[el[i].Pos[0]].Att.ArrowF == true) el[i].Att.ArrowF=true; else el[i].Att.ArrowF=false; 
          if (el[el[i].Pos[0]].Att.ArrowL == true) el[i].Att.ArrowL=true; else el[i].Att.ArrowL=false;       
          evas[el[i].Row] = elid+"_transf["+el[i].Row+"] = "+boardname+"_b.create('point', [function(){return "+elid+"_el["+el[i].Pos[0]+"].point1.X()+("+elid+"_el["+el[i].Pos[2]+"].X()-"+elid+"_el["+el[i].Pos[1]+"].X());}, function(){return "+elid+"_el["+el[i].Pos[0]+"].point1.Y()+("+elid+"_el["+el[i].Pos[2]+"].Y()-"+elid+"_el["+el[i].Pos[1]+"].Y());}] );"+elid+"_transf["+el[i].Row+"].hideElement(); "+elid+"_transf2["+el[i].Row+"] = "+boardname+"_b.create('point', [function(){return "+elid+"_el["+el[i].Pos[0]+"].point2.X()+("+elid+"_el["+el[i].Pos[2]+"].X()-"+elid+"_el["+el[i].Pos[1]+"].X());}, function(){return "+elid+"_el["+el[i].Pos[0]+"].point2.Y()+("+elid+"_el["+el[i].Pos[2]+"].Y()-"+elid+"_el["+el[i].Pos[1]+"].Y());}] );"+elid+"_transf2["+el[i].Row+"].hideElement(); "+obj_c+"('segment', ["+elid+"_transf["+el[i].Row+"], "+elid+"_transf2["+el[i].Row+"]], "+setAttributes(el[i],'strokeWidth:'+el[i].Att.Strokewidth,use_hide_el,elid)+" );";
        }        
        // FOR LINE
        else if (eval(elid+"_el["+el[i].Pos[0]+"].getType()")=='line') {          
          evas[el[i].Row] = elid+"_transf["+el[i].Row+"] = "+boardname+"_b.create('point', [function(){return "+elid+"_el["+el[i].Pos[0]+"].point1.X()+("+elid+"_el["+el[i].Pos[2]+"].X()-"+elid+"_el["+el[i].Pos[1]+"].X());}, function(){return "+elid+"_el["+el[i].Pos[0]+"].point1.Y()+("+elid+"_el["+el[i].Pos[2]+"].Y()-"+elid+"_el["+el[i].Pos[1]+"].Y());}] );"+elid+"_transf["+el[i].Row+"].hideElement(); "+elid+"_transf2["+el[i].Row+"] = "+boardname+"_b.create('point', [function(){return "+elid+"_el["+el[i].Pos[0]+"].point2.X()+("+elid+"_el["+el[i].Pos[2]+"].X()-"+elid+"_el["+el[i].Pos[1]+"].X());}, function(){return "+elid+"_el["+el[i].Pos[0]+"].point2.Y()+("+elid+"_el["+el[i].Pos[2]+"].Y()-"+elid+"_el["+el[i].Pos[1]+"].Y());}] );"+elid+"_transf2["+el[i].Row+"].hideElement(); "+obj_c+"('line', ["+elid+"_transf["+el[i].Row+"], "+elid+"_transf2["+el[i].Row+"]], "+setAttributes(el[i],'strokeWidth:'+el[i].Att.Strokewidth,use_hide_el,elid)+" );";
        }
        // FOR CIRCLES
        else if (eval(elid+"_el["+el[i].Pos[0]+"].getType()") =='circle') {
          evas[el[i].Row] = elid+"_transf["+el[i].Row+"] = "+boardname+"_b.create('point', [function(){return "+elid+"_el["+el[i].Pos[0]+"].center.X()+("+elid+"_el["+el[i].Pos[2]+"].X()-"+elid+"_el["+el[i].Pos[1]+"].X());}, function(){return "+elid+"_el["+el[i].Pos[0]+"].center.Y()+("+elid+"_el["+el[i].Pos[2]+"].Y()-"+elid+"_el["+el[i].Pos[1]+"].Y());}] );"+elid+"_transf["+el[i].Row+"].hideElement(); "+obj_c+"('circle', ["+elid+"_transf["+el[i].Row+"], function() {return "+elid+"_el["+el[i].Pos[0]+"].Radius();}], "+setAttributes(el[i],'strokeWidth:'+el[i].Att.Strokewidth,use_hide_el,elid)+" );";
        }
      }
      // * * * * * * /TRANSFORMATIONS /






      // * * * * * * IMAGES /

      else if (el[i].JsxType == 'image') {
        // Image (num1, num2, string);
         // Defines a rectangular picture image positioned in the plane at the (X, Y) location specified by (num1, num2). 
         // string specifies the URL of the image relative to the URL of the web page which contains this 
         //
        var imgWidth="", 
            imgHeight="";
        document.getElementById('hiddenimg').src = el[i].Pos[2];
        imgWidth = document.getElementById('hiddenimg').width;
        imgHeight = document.getElementById('hiddenimg').height;
        [imgWidth, imgHeight] = [imgWidth/coord_div, imgHeight/coord_div];      
        evas[el[i].Row] = obj_c+"('image', ['"+el[i].Pos[2]+"',["+el[i].Pos[0]+","+el[i].Pos[1]+"], ["+imgWidth+","+imgHeight+"]], {visible:"+el[i].Att.Visible+"} );";
      }

      // * * * * * * /IMAGES /






      // MEASUREMENTS * * * * * /

      else if (el[i].JsxType == 'length') {
        // Length(obj1, numX, numY, string);
         // Constructs a measurement of the length of segment obj1.
        evas[el[i].Row] = "function "+elid+el[i].Row+"() {return "+elid+"_el["+el[i].Pos[0]+"].L();} ";

        calc[el[i].Row] = elid+el[i].Row+"()";

        evas[el[i].Row] = evas[el[i].Row] + obj_c+'("text", ['+el[i].Pos[1]+','+el[i].Pos[2]+',function(){return "'+el[i].Pos[3] +'" + Math.round('+calc[el[i].Row]+'*'+mesaccu+')/'+mesaccu+';}], {color:"'+el[i].Att.Color+'", fixed:'+el[i].Att.Fixed+'} );';
      }
      else if (el[i].JsxType == 'angle') {
        //  Angle(obj1, obj2, obj3, numX, numY, string);
         // Constructs a measurement of the angle implied by the three points obj1, obj2, and obj3, with obj2 being the vertex of the angle. 
         // The angle will be reported as either a directed or an undirected angle, depending on the setting of the global DirectedAngles parameter.
        //evas[el[i].Row] = "function "+elid+el[i].Row+"() {if (JXG.Math.Geometry.rad("+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"],"+elid+"_el["+el[i].Pos[2]+"]) < 3.141593) return JXG.Math.Geometry.rad("+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"],"+elid+"_el["+el[i].Pos[2]+"]); else return 6.28318530718 - JXG.Math.Geometry.rad("+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"],"+elid+"_el["+el[i].Pos[2]+"]);}";
        if (directed_angles == 1)
          evas[el[i].Row] = "function "+elid+el[i].Row+"() {if (JXG.Math.Geometry.rad("+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"],"+elid+"_el["+el[i].Pos[2]+"]) < 3.141593) return JXG.Math.Geometry.rad("+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"],"+elid+"_el["+el[i].Pos[2]+"]); else return -1* (6.28318530718 - JXG.Math.Geometry.rad("+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"],"+elid+"_el["+el[i].Pos[2]+"]));}";
        else if (directed_angles == 0)
          evas[el[i].Row] = "function "+elid+el[i].Row+"() { return JXG.Math.Geometry.rad("+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"],"+elid+"_el["+el[i].Pos[2]+"]);}";
        else errors = errors + "DirectedAngles parameter is invalid!\n";
        calc[el[i].Row] = elid+el[i].Row+"()";
        evas[el[i].Row] = evas[el[i].Row] + obj_c+'("text", ['+el[i].Pos[3]+','+el[i].Pos[4]+',function(){return "'+el[i].Pos[5] +'" + Math.round('+calc[el[i].Row]+'*'+mesaccu+')/'+mesaccu+';}], {color:"'+el[i].Att.Color+'", fixed:'+el[i].Att.Fixed+'} );';
      }
      else if (el[i].JsxType == 'circumference') {
        // Circumference(obj1, numX, numY, string);
         // Constructs a measurement of the circumference of circle or circle interior obj1.
        evas[el[i].Row] = "function "+elid+el[i].Row+"() {return "+elid+"_el["+el[i].Pos[0]+"].Radius()*Math.PI*2} ";
        calc[el[i].Row] = elid+el[i].Row+"()";
          evas[el[i].Row] = evas[el[i].Row] + obj_c+'("text", ['+el[i].Pos[1]+','+el[i].Pos[2]+',function(){return "'+el[i].Pos[3]+'" + Math.round('+calc[el[i].Row]+'*'+mesaccu+')/'+mesaccu+';}], {color:"'+el[i].Att.Color+'", fixed:'+el[i].Att.Fixed+'} );';
      }
      else if (el[i].JsxType == 'radius') {
        // Radius(obj1, numX, numY, string);
         // Constructs a measurement of the radius of circle or circle interior obj1.
        evas[el[i].Row] = "function "+elid+el[i].Row+"() {return "+elid+"_el["+el[i].Pos[0]+"].Radius()} ";
        calc[el[i].Row] = elid+el[i].Row+"()";
          evas[el[i].Row] = evas[el[i].Row] + obj_c+'("text", ['+el[i].Pos[1]+','+el[i].Pos[2]+',function(){return "'+el[i].Pos[3]+'" + Math.round('+calc[el[i].Row]+'*'+mesaccu+')/'+mesaccu+';}], {color:"'+el[i].Att.Color+'", fixed:'+el[i].Att.Fixed+'} );';
      }
      else if (el[i].JsxType == 'area') {
        // Area (obj1, numX, numY, string);
         // Constructs a measurement of the area of the circle, circle interior, or polygon interior obj1. Note that the area is reported in square distance units.
        evas[el[i].Row] = "function "+elid+el[i].Row+"() {return "+elid+"_el["+el[i].Pos[0]+"].Area()} ";
        calc[el[i].Row] = elid+el[i].Row+"()";
          evas[el[i].Row] = evas[el[i].Row] + obj_c+'("text", ['+el[i].Pos[1]+','+el[i].Pos[2]+',function(){return "'+el[i].Pos[3]+'" + Math.round('+calc[el[i].Row]+'*'+mesaccu+')/'+mesaccu+';}], {color:"'+el[i].Att.Color+'", fixed:'+el[i].Att.Fixed+'} );';
      }
      else if (el[i].JsxType == 'slope') {
        // Slope (obj1, numX, numY, string);
         // Constructs a measurement of the slope of straight object obj1.
        evas[el[i].Row] = "function "+elid+el[i].Row+"() {return "+elid+"_el["+el[i].Pos[0]+"].getSlope()} ";
        calc[el[i].Row] = elid+el[i].Row+"()";
          evas[el[i].Row] = evas[el[i].Row] + obj_c+'("text", ['+el[i].Pos[1]+','+el[i].Pos[2]+',function(){return "'+el[i].Pos[3]+'" + Math.round('+calc[el[i].Row]+'*'+mesaccu+')/'+mesaccu+';}], {color:"'+el[i].Att.Color+'", fixed:'+el[i].Att.Fixed+'} );';
      }
      else if (el[i].JsxType == 'calculate') {
        // Calculate(numX, numY, prefixStr, exprStr) (objRef1, ..., objRefN);
         // Constructs a measurement of the value of a calculated expression. numX, numY, and prefixStr are the standard left and top coordinates and prefix string of every measurement object; exprStr contains a representation of the expression to calculate, and objRef1 through objRefN are object identifiers of any other measurement objects the value of which act as components of the expression.
         /// 
         // The exprStr expression is expressed in reverse Polish notation (RPN), in which arithmetic operators follow their operands. RPN eliminates the need for parentheses, because the order of evaluation is strictly left-to-right. Thus "(1+2)/3" can be phrased in RPN as "1 2 + 3 /".
         // 
         // Valid tokens within exprStr include:
         // 
         // numeric constants. Any real number can be expressed, with an optional preliminary negative sign (for negative numbers) and an optional single decimal point (for expressing non-integer values). A numeric constant may include a terminal space to delimit it in circumstances where two numeric constants are adjacent.
         // +, -, /, *. These operators perform their standard arithmetic operation: addition, subtraction, division, and multiplication.
         // ^: This operator performs exponentiation.
         // %: This operator performs unary negation.
         // @xxxx: This operator applies a predefined function. xxxx is a four-character sequence indicating the function, and may be one of the following: sin_ (sine), cos_ (cosine), tan_ (tangent), abs_ (absolute value), sqrt (square root), ln__ (natural log), rond (rounding), trnc (truncation toward zero), sgn_ (signnum), asin (arcsine), acos (arccosine), atan (arctangent), or log_ (base-10 logarithm).
         // x where x is in {A-Z}. This operand substitutes the value of another measurement in the sketch: A corresponds to measurement objRef1, B corresponds to measurement objRef2, etc. It is an error to pass an operator that does not have a corresponding objRef; i. e. to refer to C when you only supply two objRefs in the construction (implicitly, A and B).
         // #xy where x is in {A-Z} and y is in {1-9}. This operand substitutes the value of component y of a vector-based measurement x. x refers to an objRef according to the same rules above (i. e. A is objRef1, B is objRef2, etc.). Parent objRefx must be a vector-based measurement: in DR3 of JavaSketchpad, the only vector-based measurement is the Coordinate pair. y indicates the component of the vector to isolate. In DR3, the only legal values for y are 1 and 2: 1 indicates the x-component of a coordinate pair, 2 indicates the y-component of that pair.
         // Examples:
         // 
         // Calculate(10, 10, 'Sum is ','AB+C+') (5, 6, 7);
         // 
         // Calculates the sum of measurement objects #5, 6, and 7 in the sketch, and positions their sum at (10, 10) on the coordinate plane.
         // 
         // Calculate(20, 20, 'Distance is ','#A1#B1-2^#A2#B2-2^+@sqrt') (8, 9);
         // 
         // Calculates the Euclidean distance (sqrt[dX^2+dY^2]) between the two coordinate pair measurements referenced as objects #8 and 9 in the sketch.
         // 
         // Note that a space character may be used inside exprStr to delimit individual tokens. This allows you to express one plus three as '1 3+' where '13+' would be interpreted as thirteen plus (whatever preceded it).
         // 
         // Note also that the objRef list must not be empty, even if no tokens in the expression refer to object references (i. e. even if the expression's value is constant). This is a limitation of DR3, and will be fixed in the next release.
        var tmp = [];
        for (var k=0; k<el[i].Pos[4].length; k++)
          tmp[k] = el[i].Pos[4][k];
        calc[el[i].Row] = calculateFunction(el[i].Pos[3],tmp,calc,el[i].Row);
        
        evas[el[i].Row] = (obj_c+'("text", ['+el[i].Pos[0]+','+el[i].Pos[1]+',function(){return "'+el[i].Pos[2] + ' " + Math.round(('+calc[el[i].Row]+')*'+mesaccu+')/'+mesaccu+' + "'+el[i].Att.Suffix+'";}],{color:"'+el[i].Att.Color+'", fixed:'+el[i].Att.Fixed+'});');
        //if (!el[i].Att.Visible)
          //evas[el[i].Row] = evas[el[i].Row] + elid+"_el["+el[i].Row+"].hideElement();";
      }
      else if (el[i].JsxType == 'distance') {
        /// Distance(obj1, obj2, numX, numY, string);
         // Constructs a measurement of the distance between obj1 and obj2. 
         // obj2 must be a point, obj1 may be either a point or a straight object, 
         // in which case the distance is reported as the minimum (perpendicular) distance between point obj2 and the extended line containing obj1.
        if (el[el[i].Pos[0]].CType == 'points')
          evas[el[i].Row] = "function "+elid+el[i].Row+"() {return "+elid+"_el["+el[i].Pos[1]+"].Dist("+elid+"_el["+el[i].Pos[0]+"] ) * "+unit_in_pixels+"} ";
        else if (el[el[i].Pos[0]].CType == 'straightlines' || el[el[i].Pos[0]].ParentType == 'straightlines') {
          evas[el[i].Row] = "function "+elid+el[i].Row+"() {return Math.abs(("+elid+"_el["+el[i].Pos[1]+"].X()*("+elid+"_el["+el[i].Pos[0]+"].point2.Y()-"+elid+"_el["+el[i].Pos[0]+"].point1.Y())) - ("+elid+"_el["+el[i].Pos[1]+"].Y()*("+elid+"_el["+el[i].Pos[0]+"].point2.X()-"+elid+"_el["+el[i].Pos[0]+"].point1.X())) + "+elid+"_el["+el[i].Pos[0]+"].point2.X()*"+elid+"_el["+el[i].Pos[0]+"].point1.Y() - "+elid+"_el["+el[i].Pos[0]+"].point2.Y()*"+elid+"_el["+el[i].Pos[0]+"].point1.X() ) / Math.sqrt(Math.pow(("+elid+"_el["+el[i].Pos[0]+"].point2.Y()-"+elid+"_el["+el[i].Pos[0]+"].point1.Y()),2) + Math.pow(("+elid+"_el["+el[i].Pos[0]+"].point2.X()-"+elid+"_el["+el[i].Pos[0]+"].point1.X()),2)) * "+unit_in_pixels+";} ";
        }else 
          errors = errors + "distance measurement from point to "+ el[el[i].Pos[0]].CType + " is not working (Line "+el[i].Row+")!<br>";

        calc[el[i].Row] = elid+el[i].Row+"()";
          evas[el[i].Row] = evas[el[i].Row] + obj_c+'("text", ['+el[i].Pos[2]+','+el[i].Pos[3]+',function(){return "'+el[i].Pos[4] + ' " + Math.round('+calc[el[i].Row]+'*'+mesaccu+')/'+mesaccu+';}], {color:"'+el[i].Att.Color+'", fixed:'+el[i].Att.Fixed+'} );';
      }
      else if (el[i].JsxType == 'ratio/segments') {
        // Ratio/Segments (obj1, obj2, numX, numY, string);
         // Constructs a measurement of the ratio of the length of segment obj1 to the length of segment obj2. That is, this construction measures |obj|/|obj2|.
        evas[el[i].Row] = "function "+elid+el[i].Row+"() {return "+elid+"_el["+el[i].Pos[0]+"].L() / "+elid+"_el["+el[i].Pos[1]+"].L()} ";
        calc[el[i].Row] = elid+el[i].Row+"()";
          evas[el[i].Row] = evas[el[i].Row] + obj_c+'("text", ['+el[i].Pos[2]+','+el[i].Pos[3]+',function(){return "'+el[i].Pos[4] + ' " + Math.round('+calc[el[i].Row]+'*'+mesaccu+')/'+mesaccu+';}], {color:"'+el[i].Att.Color+'", fixed:'+el[i].Att.Fixed+'} );';
      }
      else if (el[i].JsxType == 'ratio/points') {
        /// Ratio/Points (obj1, obj2, obj3, numX, numY, string);
         // Constructs a measurement of the three-point ratio implied by points obj1, obj2, and obj3. 
         // If these points are A, B, and C, respectively, then the three-point ratio is |AC|/|AB|.        
        evas[el[i].Row] = "function "+elid+el[i].Row+"() {return Math.sign(Math.cos(JXG.Math.Geometry.rad("+elid+"_el["+el[i].Pos[1]+"], "+elid+"_el["+el[i].Pos[0]+"], "+elid+"_el["+el[i].Pos[2]+"])))*"+elid+"_el["+el[i].Pos[0]+"].Dist("+elid+"_el["+el[i].Pos[2]+"]) / "+elid+"_el["+el[i].Pos[0]+"].Dist("+elid+"_el["+el[i].Pos[1]+"])} ";
        calc[el[i].Row] = elid+el[i].Row+"()";
          evas[el[i].Row] = evas[el[i].Row] + obj_c+'("text", ['+el[i].Pos[3]+','+el[i].Pos[4]+',function(){return "'+el[i].Pos[5]+ ' " + Math.round('+calc[el[i].Row]+'*'+mesaccu+')/'+mesaccu+' + "'+el[i].Att.Suffix+'";}],'+setAttributes(el[i],"",use_hide_el,elid)+');'; 
      }
      // * * * * * /MEASUREMENTS /






      // BUTTONS /

      else if (el[i].JsxType == 'text') {
          if (document.getElementById('latex_fixedtext').checked == true) 
            evas[el[i].Row] = obj_c+'("text", ['+el[i].Pos[0]+','+el[i].Pos[1]+',function(){return "\\\\"+"( '+el[i].Pos[2]+' \\\\"+")";}],'+setAttributes(el[i],"",use_hide_el,elid)+');';
          else 
            evas[el[i].Row] = obj_c+'("text", ['+el[i].Pos[0]+','+el[i].Pos[1]+',function(){return "'+el[i].Pos[2]+'";}],'+setAttributes(el[i],"",use_hide_el,elid)+');';
        
      }
      else if (el[i].JsxType == 'movebutton') {
        for (var j = 0; j < el[i].Anim.length; j+=2)
          tmpstring = tmpstring +elid+"_el["+el[i].Anim[j+1]+"].moveTo(["+elid+"_el["+el[i].Anim[j]+"].X(), "+elid+"_el["+el[i].Anim[j]+"].Y()],"+el[i].Pos[2]+");";
          evas[el[i].Row] = obj_c+'("button", ['+el[i].Pos[0]+','+el[i].Pos[1]+', "'+el[i].Pos[3]+' ", function() {'+tmpstring+'} ],'+setAttributes(el[i],"",use_hide_el,elid)+');';
        el[i].Anim = tmpstring;
      }
      else if (el[i].JsxType == 'showbutton') {
        for (var j = 0; j < el[i].Anim.length; j++) {
          if (el[el[i].Anim[j]]) {
            if (el[el[i].Anim[j]].CType == 'buttons' || el[el[i].Anim[j]].CType == 'measurements')
              tmpstring = tmpstring + elid+"_el["+el[i].Anim[j]+"].showElement();";
            else if (el[el[i].Anim[j]].JsxType == 'originandunit')
              tmpstring = tmpstring + boardname +"_b.addGrid();";
            else if (el[el[i].Anim[j]].JsxType == 'locus') {
              if (el[el[el[i].Anim[j]].Pos[1]].JsxType == 'point' && curvelocus) {tmpstring = tmpstring + elid+el[el[i].Anim[j]].Row+"visible=true;"+elid+"Locus"+el[el[i].Anim[j]].Row+"("+elid+"_el["+el[el[i].Anim[j]].Pos[2]+"],"+elid+"_el["+el[el[i].Anim[j]].Pos[0]+"],"+elid+"_el["+el[el[i].Anim[j]].Pos[1]+"]);";} 
              else tmpstring = tmpstring + elid+"_el["+el[i].Anim[j]+"].setAttribute({visible:true});"; 
            }           
            else
              tmpstring = tmpstring + elid+"_el["+el[i].Anim[j]+"].setAttribute({visible:true});";    
            //if (el[i].Pos[2].indexOf("Jäljitä") >= 0 || el[i].Pos[2].indexOf("Trace") >= 0)
            if (el[el[i].Anim[j]].Opts.indexOf('traced')>=0){
              tmpstring = tmpstring + elid+"_el["+el[i].Anim[j]+"].setAttribute({trace:true});";
            }
          }
          else errors = errors + "- Error in JSP-code on line {"+el[i].Row+"}. ShowButton is trying to show element that does not excist!<br>";
        }
          evas[el[i].Row] = obj_c+'("button", ['+el[i].Pos[0]+','+el[i].Pos[1]+', "'+el[i].Pos[2]+' ", function() {'+tmpstring+'} ],'+setAttributes(el[i],"",use_hide_el,elid)+');';
          el[i].Anim = tmpstring;         
      }
      else if (el[i].JsxType == 'hidebutton') {
        for (var j = 0; j < el[i].Anim.length; j++) {
          if (el[el[i].Anim[j]]) {
            if (el[el[i].Anim[j]].CType == 'buttons' || el[el[i].Anim[j]].CType == 'measurements')
              tmpstring = tmpstring + elid+"_el["+el[i].Anim[j]+"].hideElement();";
            else if (el[el[i].Anim[j]].JsxType == 'originandunit')
              tmpstring = tmpstring + boardname +"_b.removeGrids();";
            else if (el[el[i].Anim[j]].JsxType == 'locus') {
              if (el[el[el[i].Anim[j]].Pos[1]].JsxType == 'point' && curvelocus) {tmpstring = tmpstring + elid+el[el[i].Anim[j]].Row+"visible=false;"+elid+"Locus"+el[el[i].Anim[j]].Row+"("+elid+"_el["+el[el[i].Anim[j]].Pos[2]+"],"+elid+"_el["+el[el[i].Anim[j]].Pos[0]+"],"+elid+"_el["+el[el[i].Anim[j]].Pos[1]+"]);";} 
              else tmpstring = tmpstring + elid+"_el["+el[i].Anim[j]+"].setAttribute({visible:false});"; 
            }   
            else
              tmpstring = tmpstring + elid+"_el["+el[i].Anim[j]+"].setAttribute({visible:false});";
            //if (el[i].Pos[2].indexOf("Lopeta") >= 0 || el[i].Pos[2].indexOf("Stop") >= 0) {
            if (el[el[i].Anim[j]].Opts.indexOf('traced')>=0) {
              tmpstring = tmpstring + elid+"_el["+el[i].Anim[j]+"].setAttribute({trace:false});";
              tmpstring = tmpstring + elid+"_el["+el[i].Anim[j]+"].clearTrace();";
            }
          }
          else errors = errors + "- Error in JSP-code on line {"+el[i].Row+"}. HideButton is trying to hide element that does not excist!<br>";
        }
          evas[el[i].Row] = obj_c+'("button", ['+el[i].Pos[0]+','+el[i].Pos[1]+', "'+el[i].Pos[2]+' ", function() {'+tmpstring+'} ],'+setAttributes(el[i],"",use_hide_el,elid)+');';   
        el[i].Anim = tmpstring;       
      }
      else if (el[i].JsxType == 'animatebutton') { //ANIMBUTTON
        /* AnimateButton 
           (xpos, ypos, label) (point/path pairs) 
           (speeds) (once-only flags) (direction flags); */
        evas[el[i].Row] ="";
        for (var j=0; j<el[i].Anim[0].length; j+=2) {
          tmpstring = tmpstring + 
                      "if (!anim"+el[i].Row+j+") {\n"+
                        "\tanim"+el[i].Row+j+"=true; \n"+
                        "\t"+elid+"_animOn=true; \n"+
                        "animateOn"+el[i].Row+j+"();\n"+
                      "} else {\n"+
                        "\tanim"+el[i].Row+j+" = false; \n"+
                        "\t"+elid+"_animOn=false; \n"+
                        "\tanimateOff"+el[i].Row+j+"();\n"+
                      "}";
        
        //Create anim path (corners of the board, or segment start and end point)
          if (el[i].Anim[3][j] == 1) {//If counter clockwise (on circle) or start-end-start-point on segment
            // VANHA TOIMIVA: evas[el[i].Row] = evas[el[i].Row] + "var "+elid+"_path"+el[i].Row+j+"=[];if("+elid+"_el["+el[i].Anim[0][j+1]+"].getType()=='segment'){"+elid+"_path"+el[i].Row+j+"=["+elid+"_el["+el[i].Anim[0][j+1]+"].point1,"+elid+"_el["+el[i].Anim[0][j+1]+"].point2,"+elid+"_el["+el[i].Anim[0][j+1]+"].point1];}else{"+elid+"_path"+el[i].Row+j+"=[["+boardsize[2]+","+boardsize[1]+"],["+boardsize[0]+","+boardsize[1]+"],["+boardsize[0]+","+boardsize[3]+"],["+boardsize[2]+","+boardsize[3]+"],["+boardsize[2]+","+boardsize[1]+"]];}";
            evas[el[i].Row] = evas[el[i].Row] + 
                      "function "+elid+"_path"+el[i].Row+j+"(a,b,c) {\n"+
                        "\tif(a.getType()=='segment'){\n"+
                          "\t\treturn [[a.point1.X(), a.point1.Y()], [a.point2.X(), a.point2.Y()], [a.point1.X(), a.point1.Y()]];\n"+
                        "\t}else{\n"+
                          //"\t\tvar c_x = a.center.X(), c_y = a.center.Y(); \n"+
                          //"\t\tif(c-c_y<0) b=(3.1416-b)+3.1416; \n"+
                          //"\t\treturn [[c_x+a.radius*Math.cos(b),c_y+a.radius*Math.sin(b)], \n"+
                            /*"\t\t\t[c_x+a.radius*Math.cos(b+1.5708),c_y+a.radius*Math.sin(b+1.5708)], \n"+
                            "\t\t\t[c_x+a.radius*Math.cos(b+3.1416),c_y+a.radius*Math.sin(b+3.1416)], \n"+
                            "\t\t\t[c_x+a.radius*Math.cos(b+4.7124),c_y+a.radius*Math.sin(b+4.7124)], \n"+
                            "\t\t\t[c_x+a.radius*Math.cos(b),c_y+a.radius*Math.sin(b)]]; \n"+*/
                            "return [["+boardsize[2]+",0], [0,"+boardsize[3]+"], ["+boardsize[0]+", 0], [0, "+boardsize[1]+"], ["+boardsize[2]+", 0]];"+
                        "\t}\n"+
                      "}";
          }
          else //if clockwise (on circle) or only on-way movement on segment
            evas[el[i].Row] = evas[el[i].Row] + 
                              "function "+elid+"_path"+el[i].Row+j+"(a,b,c) {\n"+
                                "\tif(a.getType()=='segment'){\n"+
                                  "\t\treturn [[a.point1.X(), a.point1.Y()], [a.point2.X(), a.point2.Y()]];\n"+
                                "\t}else{\n"+
                                  //"\t\tvar c_x = a.center.X(), c_y = a.center.Y(); \n"+
                                  //"\t\tif(c-c_y<0)b=(3.1416-b)+3.1416; \n"+
                                  /*"\t\treturn [[c_x+a.radius*3*Math.cos(b),c_y+a.radius*3*Math.sin(b)], \n"+
                                    "\t\t\t[c_x+a.radius*3*Math.cos(b-1.5708),c_y+a.radius*3*Math.sin(b-1.5708)], \n"+
                                    "\t\t\t[c_x+a.radius*3*Math.cos(b-3.1416),c_y+a.radius*3*Math.sin(b-3.1416)], \n"+
                                    "\t\t\t[c_x+a.radius*3*Math.cos(b-4.7124),c_y+a.radius*3*Math.sin(b-4.7124)], \n"+
                                    "\t\t\t[c_x+a.radius*3*Math.cos(b),c_y+a.radius*3*Math.sin(b)]]; \n"+*/
                                    "return [["+boardsize[2]+",0], [0,"+boardsize[3]+"], ["+boardsize[0]+", 0], [0, "+boardsize[1]+"], ["+boardsize[2]+", 0]];"+
                                "\t}\n"+
                              "}";
        // create variable anim[Row nr][j] and functions needed (animOn, animOff)
          if (el[i].Anim[2][j] == 0) { // if run repeatedly
            if (eval(elid+"_el["+el[i].Anim[0][j+1]+"]").getType() == 'segment')
                            evas[el[i].Row] = evas[el[i].Row] + 
                              "var anim"+el[i].Row+j+" = false; \n"+
                              "var antype"+el[i].Row+j+"="+elid+"_el["+el[i].Anim[0][j]+"].getType(); \n"+
                              "function animateOn"+el[i].Row+j+"() {\n"+
                                "\t"+elid+"_el["+el[i].Anim[0][j]+"].free(); \n"+
                                "\t"+elid+"_el["+el[i].Anim[0][j]+"].makeGlider("+elid+"_el["+el[i].Anim[0][j+1]+"]); \n"+
                                //"\t"+elid+"_el["+el[i].Anim[0][j]+"].setGliderPosition(0);\n"+
                                //"\tvar path_"+el[i].Row+"="+elid+"_path"+el[i].Row+j+"("+elid+"_el["+el[i].Anim[0][j+1]+"]); \n"+
                                "\t"+elid+"_el["+el[i].Anim[0][j]+"].moveAlong("+elid+"_path"+el[i].Row+j+"("+elid+"_el["+el[i].Anim[0][j+1]+"]), "+el[i].Anim[1][j]+", {callback: animateOn"+el[i].Row+j+"});\n"+
                                //"\tfor (var i=0; i<path_"+el[i].Row+".length; i++) {"+elid+"_el["+el[i].Anim[0][j]+"].moveTo(path_"+el[i].Row+"[i], 1000);}\n"+
                                //"var piste;"+
                                //"\twhile(anim"+el[i].Row+j+") {piste = ["+elid+"_el["+el[i].Anim[0][j+1]+"].X(),"+elid+"_el["+el[i].Anim[0][j+1]+"].Y()]; "+elid+"_el["+el[i].Anim[0][j]+"].visit([2,2], 1000);}\n"+
                              "}; \n"+
                              "function animateOff"+el[i].Row+j+"() {\n"+
                                "\t"+elid+"_el["+el[i].Anim[0][j]+"].visit([0,0], 1); \n"+
                                "\tif (antype"+el[i].Row+j+" == 'point')"+elid+"_el["+el[i].Anim[0][j]+"].free(); \n"+
                                "\tif (antype"+el[i].Row+j+" == 'glider')"+
                                  "\t\t"+elid+"_el["+el[i].Anim[0][j]+"].makeGlider("+elid+"_el["+el[el[i].Anim[0][j]].Pos[0]+"]); \n"+
                              "}; ";
            else 
              evas[el[i].Row] = evas[el[i].Row] + 
                              "var anim"+el[i].Row+j+" = false; \n"+
                              "var antype"+el[i].Row+j+"="+elid+"_el["+el[i].Anim[0][j]+"].getType(); \n"+
                              "function animateOn"+el[i].Row+j+"() {\n"+
                                "\t"+elid+"_el["+el[i].Anim[0][j]+"].free(); \n"+
                                "\t"+elid+"_el["+el[i].Anim[0][j]+"].makeGlider("+elid+"_el["+el[i].Anim[0][j+1]+"]); \n"+
                                //"\tvar path_"+el[i].Row+"="+elid+"_path"+el[i].Row+j+"("+elid+"_el["+el[i].Anim[0][j+1]+"], Math.acos(("+elid+"_el["+el[i].Anim[0][j]+"].X()-"+elid+"_el["+el[i].Anim[0][j+1]+"].center.X())/"+elid+"_el["+el[i].Anim[0][j+1]+"].radius), "+elid+"_el["+el[i].Anim[0][j]+"].Y()); \n"+
                                //"\t"+elid+"_el["+el[i].Anim[0][j]+"].moveAlong("+elid+"_path"+el[i].Row+j+"("+elid+"_el["+el[i].Anim[0][j+1]+"], Math.acos(("+elid+"_el["+el[i].Anim[0][j]+"].X()-"+elid+"_el["+el[i].Anim[0][j+1]+"].center.X())/"+elid+"_el["+el[i].Anim[0][j+1]+"].radius), "+elid+"_el["+el[i].Anim[0][j]+"].Y()), "+el[i].Anim[1][j]+", {callback: animateOn"+el[i].Row+j+"});\n"+
                                "\t"+elid+"_el["+el[i].Anim[0][j]+"].moveAlong("+elid+"_path"+el[i].Row+j+"("+elid+"_el["+el[i].Anim[0][j+1]+"]), "+el[i].Anim[1][j]+", {callback: animateOn"+el[i].Row+j+"});\n"+
                                //"\t"+elid+"_el["+el[i].Anim[0][j]+"].startAnimation(1,100); \n"+
                                //"\t"+elid+"_el["+el[i].Anim[0][j]+"].moveTo(path_"+el[i].Row+"[i], 1000); \n"+
                                //"\t for(var i=0; i<4; i++) {"+elid+"_el["+el[i].Anim[0][j]+"].slideObject(path_"+el[i].Row+"[i])} \n"+
                                //"\tfor (var i=0; i<path_"+el[i].Row+".length; i++) {"+elid+"_el["+el[i].Anim[0][j]+"].moveTo(path_"+el[i].Row+"[i], 1000);}\n"+
                              "}; \n"+
                              "function animateOff"+el[i].Row+j+"() {\n"+
                                "\t"+elid+"_el["+el[i].Anim[0][j]+"].visit([0,0], 1); \n"+
                                "\tif (antype"+el[i].Row+j+" == 'point')"+elid+"_el["+el[i].Anim[0][j]+"].free(); \n"+
                                "\tif (antype"+el[i].Row+j+" == 'glider')"+
                                  "\t\t"+elid+"_el["+el[i].Anim[0][j]+"].makeGlider("+elid+"_el["+el[el[i].Anim[0][j]].Pos[0]+"]); \n"+
                              "}; ";
                              //  elid+"_el["+el[i].Anim[0][j+1]+"].on('drag', function(){if("+elid+"_animOn)var path_"+el[i].Row+"="+elid+"_path"+el[i].Row+j+"("+elid+"_el["+el[i].Anim[0][j+1]+"], Math.acos(("+elid+"_el["+el[i].Anim[0][j]+"].X()-"+elid+"_el["+el[i].Anim[0][j+1]+"].center.X())/"+elid+"_el["+el[i].Anim[0][j+1]+"].radius), "+elid+"_el["+el[i].Anim[0][j]+"].Y()); \n"+
                              //  "\t"+elid+"_el["+el[i].Anim[0][j]+"].moveAlong(path_"+el[i].Row+", "+el[i].Anim[1][j]+", {callback: animateOn"+el[i].Row+j+"});});";
            }
          else { // if run only once
            //evas[el[i].Row] = evas[el[i].Row] + "var anim"+el[i].Row+j+" = false; var antype"+el[i].Row+j+"="+elid+"_el["+el[i].Anim[0][j]+"].getType(); function animateOn"+el[i].Row+j+"() {"+elid+"_el["+el[i].Anim[0][j]+"].makeGlider("+elid+"_el["+el[i].Anim[0][j+1]+"]); "+elid+"_el["+el[i].Anim[0][j]+"].moveAlong("+elid+"_path"+el[i].Row+j+", "+el[i].Anim[1][j]+");}; function animateOff"+el[i].Row+j+"() {"+elid+"_el["+el[i].Anim[0][j]+"].visit([0,0], 1); if (antype"+el[i].Row+j+" == 'point')"+elid+"_el["+el[i].Anim[0][j]+"].free(); if (antype"+el[i].Row+j+" == 'glider')"+elid+"_el["+el[i].Anim[0][j]+"].makeGlider("+elid+"_el["+el[el[i].Anim[0][j]].Pos[0]+"]);}; ";
            if (eval(elid+"_el["+el[i].Anim[0][j+1]+"]").getType() == 'segment')
                evas[el[i].Row] = evas[el[i].Row] + 
                              "var anim"+el[i].Row+j+" = false; \n"+
                              "var antype"+el[i].Row+j+"="+elid+"_el["+el[i].Anim[0][j]+"].getType(); \n"+
                              "function animateOn"+el[i].Row+j+"() {\n"+
                                "\t"+elid+"_el["+el[i].Anim[0][j]+"].makeGlider("+elid+"_el["+el[i].Anim[0][j+1]+"]); \n"+
                                "\tvar path_"+el[i].Row+"="+elid+"_path"+el[i].Row+j+"("+elid+"_el["+el[i].Anim[0][j+1]+"]); \n"+
                                "\t"+elid+"_el["+el[i].Anim[0][j]+"].moveAlong(path_"+el[i].Row+", "+el[i].Anim[1][j]+");\n"+
                              "}; \n"+
                              "function animateOff"+el[i].Row+j+"() {\n"+
                                "\t"+elid+"_el["+el[i].Anim[0][j]+"].visit([0,0], 1); \n"+
                                "\tif (antype"+el[i].Row+j+" == 'point')"+elid+"_el["+el[i].Anim[0][j]+"].free(); \n"+
                                "\tif (antype"+el[i].Row+j+" == 'glider')"+
                                  "\t\t"+elid+"_el["+el[i].Anim[0][j]+"].makeGlider("+elid+"_el["+el[el[i].Anim[0][j]].Pos[0]+"]); \n"+
                              "}; ";
            else 
              evas[el[i].Row] = evas[el[i].Row] + 
                              "var anim"+el[i].Row+j+" = false; \n"+
                              "var antype"+el[i].Row+j+"="+elid+"_el["+el[i].Anim[0][j]+"].getType(); \n"+
                              "function animateOn"+el[i].Row+j+"() {\n"+
                                "\t"+elid+"_el["+el[i].Anim[0][j]+"].makeGlider("+elid+"_el["+el[i].Anim[0][j+1]+"]); \n"+
                                //"\tvar path_"+el[i].Row+"="+elid+"_path"+el[i].Row+j+"("+elid+"_el["+el[i].Anim[0][j+1]+"], Math.acos(("+elid+"_el["+el[i].Anim[0][j]+"].X()-"+elid+"_el["+el[i].Anim[0][j+1]+"].center.X())/"+elid+"_el["+el[i].Anim[0][j+1]+"].radius), "+elid+"_el["+el[i].Anim[0][j]+"].Y()); "+
                                //"\t"+elid+"_el["+el[i].Anim[0][j]+"].moveAlong(path_"+el[i].Row+", "+el[i].Anim[1][j]+");\n"+
                                "\t"+elid+"_el["+el[i].Anim[0][j]+"].moveAlong("+elid+"_path"+el[i].Row+j+"("+elid+"_el["+el[i].Anim[0][j+1]+"]), "+el[i].Anim[1][j]+", {callback: animateOn"+el[i].Row+j+"});\n"+                              
                              "}; \n"+
                              "function animateOff"+el[i].Row+j+"() {\n"+
                                "\t"+elid+"_el["+el[i].Anim[0][j]+"].visit([0,0], 1); \n"+
                                "\tif (antype"+el[i].Row+j+" == 'point')"+elid+"_el["+el[i].Anim[0][j]+"].free(); \n"+
                                "\tif (antype"+el[i].Row+j+" == 'glider')"+elid+"_el["+el[i].Anim[0][j]+"].makeGlider("+elid+"_el["+el[el[i].Anim[0][j]].Pos[0]+"]); \n"+
                              "}; ";
          }
        }           
          if (animtype.indexOf("animtype1")>=0) evas[el[i].Row] = evas[el[i].Row] + 'function click'+el[i].Row+'() {'+tmpstring+'} '+obj_c+'("button", ['+el[i].Pos[0]+','+el[i].Pos[1]+', "'+el[i].Pos[2]+'", function() { click'+el[i].Row+'();}], {fixed:true} );'; 
          
          // if user selected 'animtype2' the this
          else {
            evas[el[i].Row] = evas[el[i].Row] + obj_c+'("button", ['+el[i].Pos[0]+','+el[i].Pos[1]+', "'+el[i].Pos[2]+' ", function() {if ('+elid+'_animOn){'+elid+'_el['+el[i].Anim[0][0]+'].stopAnimation(); '+elid+'_animOn=false;} else{'+elid+'_el['+el[i].Anim[0][0]+'].startAnimation(1,'+animticks+'); '+elid+'_animOn=true;}}], {fixed:true, showInfoBox:'+el[i].Att.Infobox+'});'; 
            if (eval(elid+"_el["+el[i].Anim[0][0]+"]").getType()!="glider")
              errors = errors + "Error on line ["+el[i].Row+"]: You can't use animation2 for other objects than 'Point on object'. <br>Some animation may not work properly.<br><br>";
          }
        el[i].Anim = tmpstring;      
      } //ANIMBUTTON

      else if (el[i].JsxType == 'simultaneousbutton') {
        for (var j = 0; j < el[i].Anim.length; j++) {
            tmpstring = " "+tmpstring + el[el[i].Anim[j]].Anim;
        }
          evas[el[i].Row] = obj_c+'("button", ['+el[i].Pos[0]+','+el[i].Pos[1]+', "'+el[i].Pos[2]+' ", function(){'+tmpstring+'} ],'+setAttributes(el[i],"",use_hide_el,elid)+');';
        el[i].Anim = tmpstring;
      }

      // * * * * * * /BUTTONS 






      // COORDINATE SYSTEM *  * * * /

        else if(el[i].JsxType == 'originandunit') {

          evas[el[i].Row] = elid+"_el["+el[i].Row+"] = ["+elid+"_el["+el[i].Pos[0]+"],function(){return "+elid+"_el["+el[i].Pos[0]+"].Dist("+elid+"_el["+el[i].Pos[1]+"]);}];" 
          
          evas[el[i].Row] = evas[el[i].Row] + boardname+'_b.moveOrigin('+scalex * getOrigin(sketch).slice(0,getOrigin(sketch).indexOf(','))+','+scaley * getOrigin(sketch).slice(getOrigin(sketch).indexOf(',')+1,getOrigin(sketch).length)+');';
          
        }
        else if(el[i].JsxType == 'coordinates') {
          /*Coordinates(obj1, obj2, numX, numY, string);
            Constructs a measurement of the ordered pair of coordinates representing point obj1's location with respect to the coordinate system obj2.
            https://www.math.uni-bielefeld.de/~lisken/cgi-bin/jsp_qref?5.7.11+5.9*+5.7
            */
          calc[el[i].Row] = ["function(){return "+elid+"_el["+el[i].Pos[0]+"].X();}", "function(){return "+elid+"_el["+el[i].Pos[0]+"].Y();}"];
          evas[el[i].Row] = obj_c+"('text', ["+el[i].Pos[2]+","+el[i].Pos[3]+",function(){return '"+el[i].Pos[4]+" ('+ Math.round("+elid+"_el["+el[i].Pos[0]+"].X()*"+mesaccu+")/"+mesaccu+" +','+ Math.round("+elid+"_el["+el[i].Pos[0]+"].Y()*"+mesaccu+")/"+mesaccu+" +')';}], {name:'"+el[i].Att.Name+"',color:'"+el[i].Att.Color+"', visible:"+el[i].Att.Visible+", anchorX:'"+el[i].Att.AnchorX+"', fixed:"+el[i].Att.Fixed+", showInfoBox:"+el[i].Att.Infobox+", anchorX:'left'} );";

        }
        // * * * * * /COORDINATE SYSTEM /






        // LOCUS * * * * * * /
        else if (el[i].JsxType == 'locus') {
          // First let's check if Locus is replaced
          // el.locusToReplace = row of the locus
          // el.replaceLocus = replace function as string "function(x){something...}"
          
          if(el[i].JSPComment.indexOf('locus')>=0) {
            try {
              var lrtmp1 = lrtmp2 = ''; 
              while (el[i].replaceLocus.indexOf('@')>0) {
                lrtmp1 = el[i].replaceLocus.slice(el[i].replaceLocus.indexOf('@'), el[i].replaceLocus.indexOf('@', el[i].replaceLocus.indexOf('@')+1)+1);
                //alert(lrtmp1);
                if (calc[eval(lrtmp1.slice(1,lrtmp1.length-1))])
                  lrtmp2 = calc[eval(lrtmp1.slice(1,lrtmp1.length-1))];
                else lrtmp2 = elid + "_el[" + lrtmp1.slice(1,lrtmp1.length-1) +"]";
                //alert(lrtmp2);
                el[i].replaceLocus = el[i].replaceLocus.replace(lrtmp1, lrtmp2);
                el[i].replaceLocus += ', ' + setAttributes(el[i],"",use_hide_el,elid);
              } 
              //var refe1 = elid + el[i].replaceLocus.slice(el[i].replaceLocus.lastIndexOf(',', el[i].replaceLocus.indexOf(';'))+1, el[i].replaceLocus.indexOf(';'));
              //if (refe1.indexOf(')')>=0) {refe1 = refe1.slice(0,refe1.indexOf(')'));
              //  el[i].replaceLocus = el[i].replaceLocus.slice(0, el[i].replaceLocus.lastIndexOf(',',el[i].replaceLocus.indexOf(';'))+1) + refe1 + "()" + el[i].replaceLocus.slice(el[i].replaceLocus.indexOf(';')-1, el[i].replaceLocus.length-1);
              //}
              //else {
              //  el[i].replaceLocus = el[i].replaceLocus.slice(0, el[i].replaceLocus.lastIndexOf(',',el[i].replaceLocus.indexOf(';'))+1) + refe1 + "()" + el[i].replaceLocus.slice(el[i].replaceLocus.indexOf(';'), el[i].replaceLocus.length-1);
              //}
              //el[i].replaceLocus = el[i].replaceLocus.slice(0, el[i].replaceLocus.lastIndexOf(',',el[i].replaceLocus.indexOf(';'))+1) + refe1 + "()" + el[i].replaceLocus.slice(el[i].replaceLocus.indexOf(';')-1, el[i].replaceLocus.length-1);
              //alert(el[i].replaceLocus);
              //evas[el[i].Row] = obj_c+"('functiongraph', "+el[i].replaceLocus+ ");"
              evas[el[i].Row] = obj_c+"( "+el[i].replaceLocus+ ");"
            } catch(err) {alert(err.message + err);}
          } else {
            // JSP description: Locus(obj1, obj2, obj3, num);
              //Constructs the locus of obj1 as point obj2 moves along the path determined by obj3. 
              //JavaSketchpad devotes num samples to the representation of the locus--higher values of 
              //num correspond to more accurate approximations of the locus, but take more time and computer memory to process.
            var usertick = 1/30; //set user ticks here
            //if (el[i].Opts.indexOf('ticks(')> 0) usertick = eval(el[i].Opts.slice(el[i].Opts.indexOf('ticks(')+6, el[i].Opts.indexOf(')',el[i].Opts.indexOf('ticks(')+6)));
            //usertick = 1/usertick;
            if (el[i].Pos[3] <= 20) usertick = 1/eval(el[i].Pos[3]);
            else usertick = 1/((eval(el[i].Pos[3]) / 21) + 20); 

            // If locus is created by path of a glider, locus will be created as a tracecurve construction in JSXGraph
            if (eval(elid+"_el["+el[i].Pos[1]+"]").getType() == 'glider')
              evas[el[i].Row] = obj_c+"('tracecurve', ["+elid+"_el["+el[i].Pos[1]+"],"+elid+"_el["+el[i].Pos[0]+"]], {name:'"+el[i].Att.Name+"', strokeColor:'"+el[i].Att.Color+"', visible:"+el[i].Att.Visible+", numberPoints:"+el[i].Pos[3]+", highlight:false} );";
            
            
            else if (eval(elid+"_el["+el[i].Pos[1]+"]").getType() == 'point') {
                
                locus_counter = locus_counter +1;
                var lrobj = [];
                var end_gh_r = eval(el[i].Pos[0]);
                var k = eval(el[i].Pos[1]);
                var need_to_duplicate = [];
                var tmp_listnr = 0;
                var tmp_duplicate = "";
                var tmp_duplicates = [];
                var tmp_tmp = "";
                var numPoints = eval(el[i].Pos[3]);
                    if (numPoints<50) numPoints = 50;
                    else if (numPoints>100) numPoints = 100;

              if (el[el[i].Pos[2]].JsxType == 'circle') {
                
                if (!curvelocus) {

                  lrobj.push(eval(el[i].Pos[1])); //starting row to check + duplicate
                  //alert(lrobj[0] +","+  end_gh_r);
                  need_to_duplicate.push(eval(el[i].Pos[1]));
                  
                  while(k < end_gh_r) {
                    k = k+1;
                    for (var j=0; j<need_to_duplicate.length; j++) {
                      if (evas[el[k].Row].indexOf(elid+"_el["+need_to_duplicate[j]+"]")>=0 && need_to_duplicate.indexOf(k)<0) {
                        need_to_duplicate.push(k);
                      }
                    }
                  } // here is the list for lines needed to duplicate : alert(need_to_duplicate);

                  if (el[need_to_duplicate[0]].JsxType == 'point')
                    tmp_duplicates.push( "var " + elid + "_dup"+locus_counter+" = [];\n"+ 
                                      elid + "_dup"+locus_counter+"["+need_to_duplicate[0]+"] = " + boardname + "_b.create('glider', ["+elid+"_el["+el[i].Pos[2]+"]], );\n"+ elid + "_dup"+locus_counter+"["+need_to_duplicate[0]+"].hideElement();");
                  else {alert("Something went wrong in Locus operation!"); return null;}

                  for (var j = 1; j<need_to_duplicate.length; j++) {
                    tmp_duplicate = evas[need_to_duplicate[j]];
                    for (var m=0; m<need_to_duplicate.length; m++) {
                      while(tmp_duplicate.indexOf(elid+"_el["+need_to_duplicate[m]+"]")>=0)
                        tmp_duplicate = tmp_duplicate.replace(elid+"_el["+need_to_duplicate[m]+"]", elid + "_dup"+locus_counter+"["+need_to_duplicate[m]+"]");
                      while(tmp_duplicate.indexOf(elid + "_dup"+locus_counter+"["+need_to_duplicate[m]+"].hideElement();")>=0)
                        tmp_duplicate = tmp_duplicate.replace(elid + "_dup"+locus_counter+"["+need_to_duplicate[m]+"].hideElement();", "\n");
                      
                      if (tmp_duplicate.indexOf('],{')>0) {
                        tmp_tmp = tmp_duplicate.slice(tmp_duplicate.indexOf('],{')+2, tmp_duplicate.lastIndexOf('}')+1);
                        tmp_duplicate = tmp_duplicate.replace(tmp_tmp, " {} ");
                      }                   
                    }
                    tmp_duplicates.push(tmp_duplicate + elid + "_dup"+locus_counter+"["+need_to_duplicate[j]+"].hideElement()");
                  }    
                  evas[el[i].Row] = "";
                  for (var j=0; j<tmp_duplicates.length; j++) {
                    evas[el[i].Row] = evas[el[i].Row] + tmp_duplicates[j] + "\n";
                  } 
                  evas[el[i].Row] = evas[el[i].Row] + obj_c + "('tracecurve', ["+elid+"_dup"+locus_counter+"["+el[i].Pos[1]+"],"+elid+"_dup"+locus_counter+"["+el[i].Pos[0]+"]], {name:'"+el[i].Att.Name+"', strokeColor:'"+el[i].Att.Color+"', visible:"+el[i].Att.Visible+", numberPoints:"+numPoints+", highlight:false} );";
                }


                // If locus is created by path that is circle or segment, and a point other than glider mover along that path, locus will be created by this algorithm
                // Algorithm creates locus like explained next:
                // - make the point to be a gliderpoint on wanted path
                // - move point along that path distance of a tick (user can define, default is 30 ticks)
                // - while point is moving along path, collect the coordinatedata of the point we want to draw the locus
                // - clear all old segments if there are any, and create segments between all the coordinatepoint from the data
                // - make point not to be gliderpoint anymore, and return it to its startingpoint
                // - make events for the nessessary objects to launch this function
                else {                       
                  // vanhja "toimiva"
                  //evas[el[i].Row] = "var "+elid+el[i].Row+"visible="+el[i].Att.Visible+";"+ elid+"_el["+el[i].Row+"]=[]; function "+elid+"Locus"+el[i].Row+"(obj1,obj2,obj3){var objstart=[obj3.X(),obj3.Y()];var locuspath=[];var tick=0;obj3.makeGlider(obj1);while(tick<=1.02){obj3.setGliderPosition(tick); locuspath.push([obj2.X(),obj2.Y()]); tick=tick+"+usertick+";} for(var i=0; i<locuspath.length; i++){"+boardname+"_b.removeObject("+elid+"_el["+el[i].Row+"][i]); if(i==0)"+elid+"_el["+el[i].Row+"][i]="+boardname+"_b.create('segment',[locuspath[i],locuspath[locuspath.length-1]],{strokeWidth:1,fixed:true,visible:"+elid+el[i].Row+"visible}); else "+elid+"_el["+el[i].Row+"][i]="+boardname+"_b.create('segment',[locuspath[i-1],locuspath[i]],{strokeWidth:1,fixed:true, visible:"+elid+el[i].Row+"visible}); } if(!"+elid+"_animOn){obj3.free(); obj3.setPosition(JXG.COORDS_BY_USER, objstart); "+boardname+"_b.update();} return locuspath; }"; 
                  evas[el[i].Row] = "var "+elid+el[i].Row+"visible="+el[i].Att.Visible+";"+ elid+"_el["+el[i].Row+"]=[]; \n"+
                                    "function "+elid+"Locus"+el[i].Row+"(obj1,obj2,obj3){\n\t"+
                                      "var objstart=[obj3.X(),obj3.Y()],\n\t"+
                                      "\tlocuspathx=[], locuspathy=[],\n\t"+
                                      "\ttick=0;\n\t"+
                                      "obj3.makeGlider(obj1);\n\t"+
                                      "while(tick<=1.00){\n\t\t"+
                                        "obj3.setGliderPosition(tick); \n\t\t"+
                                        "locuspathx.push(obj2.X()); \n\t\t"+
                                        "locuspathy.push(obj2.Y()); \n\t\t"+
                                        "tick=tick+"+usertick+";} \n\t"+
                                      boardname+"_b.removeObject("+elid+"_el["+el[i].Row+"][0]); \n\t"+
                                      elid+"_el["+el[i].Row+"][0]="+boardname+"_b.create('curve',[locuspathx, locuspathy],{doAdvancedPlot:false,numberPointsLow:10,numberPointsHigh:10, strokeWidth:1,fixed:true,visible:"+elid+el[i].Row+"visible});  \n\t"+
                                      "if(!"+elid+"_animOn){\n\t\t"+
                                        "obj3.free(); \n\t\t"+
                                        "obj3.setPosition(JXG.COORDS_BY_USER, objstart); \n\t\t"+
                                        boardname+"_b.update();\n\t} \n"+
                                    "}"; 
                                    //alert(evas[el[i].Row]);
                  evas[el[i].Row] = evas[el[i].Row] + elid+"Locus"+el[i].Row+"("+elid+"_el["+el[i].Pos[2]+"],"+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"]);\n";
                  evas[el[i].Row] = evas[el[i].Row] + 
                                    elid+"_el["+el[i].Pos[2]+"].on('up', function() {\n"+
                                    "\t"+ elid+"Locus"+el[i].Row+"("+elid+"_el["+el[i].Pos[2]+"],"+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"]);\n});\n"+
                                    elid+"_el["+el[i].Pos[2]+"].center.on('up', function() {\n"+
                                    "\t"+ elid+"Locus"+el[i].Row+"("+elid+"_el["+el[i].Pos[2]+"],"+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"]);\n});";
                  if (el[el[i].Pos[2]].JsxType == 'circle') {                  
                                    evas[el[i].Row] = evas[el[i].Row] + 
                                    elid+"_el["+el[el[el[i].Pos[2]].Pos[1]].Row+"].on('up', function() {\n"+
                                    "\t"+ elid+"Locus"+el[i].Row+"("+elid+"_el["+el[i].Pos[2]+"],"+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"]);\n});";
                  }
                }
              }
              else if (el[el[i].Pos[2]].JsxType == 'segment') {
                
                if (!curvelocus) {
                  lrobj.push(eval(el[i].Pos[1])); //starting row to check + duplicate
                  //alert(lrobj[0] +","+  end_gh_r);
                  need_to_duplicate.push(eval(el[i].Pos[1]));

                  while(k < end_gh_r) {
                    k = k+1;
                    for (var j=0; j<need_to_duplicate.length; j++) {
                      if (evas[el[k].Row].indexOf(elid+"_el["+need_to_duplicate[j]+"]")>=0 && need_to_duplicate.indexOf(k)<0) {
                        need_to_duplicate.push(k);
                      }
                    }
                  } // here is the list for lines needed to duplicate : alert(need_to_duplicate);

                  if (el[need_to_duplicate[0]].JsxType == 'point')
                    tmp_duplicates.push( "var " + elid + "_dup"+locus_counter+" = [];\n"+ 
                                      elid + "_dup"+locus_counter+"["+need_to_duplicate[0]+"] = " + boardname + "_b.create('glider', ["+elid+"_el["+el[i].Pos[2]+"]], );\n"+ elid + "_dup"+locus_counter+"["+need_to_duplicate[0]+"].hideElement();");
                  else {alert("Something went wrong in Locus operation!"); return null;}

                  for (var j = 1; j<need_to_duplicate.length; j++) {
                    tmp_duplicate = evas[need_to_duplicate[j]];
                    for (var m=0; m<need_to_duplicate.length; m++) {
                      while(tmp_duplicate.indexOf(elid+"_el["+need_to_duplicate[m]+"]")>=0)
                        tmp_duplicate = tmp_duplicate.replace(elid+"_el["+need_to_duplicate[m]+"]", elid + "_dup"+locus_counter+"["+need_to_duplicate[m]+"]");
                      while(tmp_duplicate.indexOf(elid + "_dup"+locus_counter+"["+need_to_duplicate[m]+"].hideElement();")>=0)
                        tmp_duplicate = tmp_duplicate.replace(elid + "_dup"+locus_counter+"["+need_to_duplicate[m]+"].hideElement();", "\n");
                      
                      if (tmp_duplicate.indexOf('],{')>0) {
                        tmp_tmp = tmp_duplicate.slice(tmp_duplicate.indexOf('],{')+2, tmp_duplicate.lastIndexOf('}')+1);
                        tmp_duplicate = tmp_duplicate.replace(tmp_tmp, " {} ");
                      }                   
                    }
                    tmp_duplicates.push(tmp_duplicate + elid + "_dup"+locus_counter+"["+need_to_duplicate[j]+"].hideElement()");
                  }    
                  evas[el[i].Row] = "";
                  for (var j=0; j<tmp_duplicates.length; j++) {
                    evas[el[i].Row] = evas[el[i].Row] + tmp_duplicates[j] + "\n";
                  } 
                  evas[el[i].Row] = evas[el[i].Row] + obj_c + "('tracecurve', ["+elid+"_dup"+locus_counter+"["+el[i].Pos[1]+"],"+elid+"_dup"+locus_counter+"["+el[i].Pos[0]+"]], {name:'"+el[i].Att.Name+"', strokeColor:'"+el[i].Att.Color+"', visible:"+el[i].Att.Visible+", numberPoints:"+numPoints+", highlight:false} );";
                }

                else {
                  //vanha "toimiva"
                  //evas[el[i].Row] = "var "+elid+el[i].Row+"visible="+el[i].Att.Visible+";"+ elid+"_el["+el[i].Row+"]=[]; function "+elid+"Locus"+el[i].Row+"(obj1,obj2,obj3){var objstart=[obj3.X(),obj3.Y()];var locuspath=[];var tick=0;obj3.makeGlider(obj1);while(tick<=1.02){obj3.setGliderPosition(tick); locuspath.push([obj2.X(),obj2.Y()]); tick=tick+"+usertick+";} for(var i=0; i<locuspath.length; i++){"+boardname+"_b.removeObject("+elid+"_el["+el[i].Row+"][i]); if(i==0)"+elid+"_el["+el[i].Row+"][i]="+boardname+"_b.create('segment',[locuspath[i],locuspath[i]],{strokeWidth:1,fixed:true,visible:"+elid+el[i].Row+"visible}); else "+elid+"_el["+el[i].Row+"][i]="+boardname+"_b.create('segment',[locuspath[i-1],locuspath[i]],{strokeWidth:1,fixed:true, visible:"+elid+el[i].Row+"visible}); } if(!"+elid+"_animOn){obj3.free(); obj3.setPosition(JXG.COORDS_BY_USER, objstart); "+boardname+"_b.update();} return locuspath; }"; 
                  evas[el[i].Row] = "var "+elid+el[i].Row+"visible="+el[i].Att.Visible+";"+ elid+"_el["+el[i].Row+"]=[]; \n"+
                                    "function "+elid+"Locus"+el[i].Row+"(obj1,obj2,obj3){\n"+
                                      "\tvar objstart=[obj3.X(),obj3.Y()],\n\t"+
                                      "\tlocuspathx=[], locuspathy=[],\n"+
                                      "\ttick=0;\n"+
                                      "\tobj3.makeGlider(obj1);\n"+
                                      "\tobj3.setGliderPosition(0);\n"+
                                      "\tlocuspathx.push(obj2.X());locuspathy.push(obj2.Y());\n"+
                                      "\tobj3.setGliderPosition(1);\n"+
                                      "\tlocuspathx.push(obj2.X());locuspathy.push(obj2.Y());\n"+                                                                       
                                      "\t"+boardname+"_b.removeObject("+elid+"_el["+el[i].Row+"][0]); \n"+
                                      "\t"+elid+"_el["+el[i].Row+"][0]="+boardname+"_b.create('curve',[locuspathx,locuspathy],{strokeWidth:1,fixed:true,visible:"+elid+el[i].Row+"visible}); \n"+
                                      "\tif(!"+elid+"_animOn){\n"+
                                      "\t\tobj3.free(); \n"+
                                      "\t\tobj3.setPosition(JXG.COORDS_BY_USER, objstart); \n"+
                                      "\t\t"+boardname+"_b.update();\n"+
                                      "} \n"+
                                      "}"; 
                  evas[el[i].Row] = evas[el[i].Row] + elid+"Locus"+el[i].Row+"("+elid+"_el["+el[i].Pos[2]+"],"+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"]);";
                  evas[el[i].Row] = evas[el[i].Row] + 
                                    elid+"_el["+el[i].Pos[2]+"].on('up', function() {\n"+ 
                                      "\t"+elid+"Locus"+el[i].Row+"("+elid+"_el["+el[i].Pos[2]+"],"+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"]);\n"+
                                    "});\n"+ 
                                    elid+"_el["+el[i].Pos[2]+"].point1.on('up', function() {\n"+ 
                                      "\t"+elid+"Locus"+el[i].Row+"("+elid+"_el["+el[i].Pos[2]+"],"+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"]);\n"+
                                    "});\n"+ 
                                    elid+"_el["+el[i].Pos[2]+"].point2.on('up', function(){\n"+ 
                                      "\t"+elid+"Locus"+el[i].Row+"("+elid+"_el["+el[i].Pos[2]+"],"+elid+"_el["+el[i].Pos[0]+"],"+elid+"_el["+el[i].Pos[1]+"]);\n"+
                                    "});";
                }
              }

              else errors = errors + "Traced point [line:"+el[i].Pos[1]+"] is not a type 'Point on object' [type is "+el[el[i].Pos[1]].Type+"].<br> 'Locus' can be constructed for a 'Point'-element only if path [line:"+el[i].Pos[2]+"] in 'Circle' or 'Segment'.<br>Locus [line:"+el[i].Row+"] was not constructed!<br><br>";
            }
            else 
              //evas[el[i].Row] = obj_c+"('locus', ["+elid+"_el["+el[i].Pos[0]+"]], {name:'"+el[i].Att.Name+"', strokeColor:'"+el[i].Att.Color+"', visible:"+el[i].Att.Visible+"} );";
              errors = errors + "Error on line ["+el[i].Row+"]: Traced point on line ["+el[i].Pos[1]+"] is not a type 'Point on object'.<br> This locus may not work properly. <br><br>";
          }
        }

        // * * * * * /LOCUS /

        




        // ********* / MACROS /

        else if (jsp_macros.indexOf(el[i].Type) >= 0) {
          tmp = "";
          for (var k in el[i].Pos) {
            tmp = tmp + elid + "_el[" + el[i].Pos[k] + "],";
          }
          tmp = tmp.slice(0,tmp.lastIndexOf(','));
          //tmp = tmp +  '"' + setAttributes(el[i], '') + '"';
          evas[el[i].Row] = elid+"_el["+el[i].Row+"] = " + el[i].Type + "(" + tmp + ");\n" + elid+"_el["+ el[i].Row +"].setAttribute("+setAttributes(el[i], '')+");";

          //alert(evas[el[i].Row]);
        }

        else {
          errors = errors + "Operator '"+el[i].Type+"' is unknown or is not supported yet.<br> Converting stopped on line ["+el[i].Row+"]. <br><br>";
          //errors = errors + el[i].Original_line + "\n\n";
          //errors = errors + "\n" + document.getElementById(code_input).value.slice(document.getElementById(code_input).value.lastIndexOf(';', el[i].att), document.getElementById(code_input).value.indexOf(';'), el[i].att ) + " might also be bad $JSP name for element.";
          break;
        }

        
        if (el[i].Macro.length > 0) {
          tmp = "";
          //alert('toimii tähän saakka');
          for (var j = 1; j<el[i].Macro.length; j++) {
            //alert(el[i].Macro[j].Row);
            if (j<el[i].Macro.length-1) { 
              el[i].Macro[j] = "\tvar " + el[i].Macro[0].slice(0,3) + "_" + el[i].Macro[j].Row + " = " + boardname + "_b.create" + getMacroFunction(el[i].Macro[j]) + "\n"; 
              tmp = tmp + el[i].Macro[j];
            }
            else el[i].Macro[j] = "\treturn " + document.getElementById('jsxgboardname').value + "_b.create" + getMacroFunction(el[i].Macro[j]) + "\n";            
          }
          el[i].Macro[0] = "function " + el[i].Macro[0] + " { \n";
          el[i].Macro[el[i].Macro.length-1] = el[i].Macro[el[i].Macro.length-1] + "}\n";
          tmp = el[i].Macro[0] + tmp + el[i].Macro[el[i].Macro.length-1];
          evas[el[i].Row] = tmp + evas[el[i].Row];
        }
        

        //Hide buttons and measurements if they are 'hidden' in options field
        if (!el[i].Att.Visible && (el[i].CType == 'measurements' || el[i].CType == 'buttons') ) {evas[el[i].Row] = evas[el[i].Row] + elid+"_el["+el[i].Row+"].hideElement();";}        

        

        //update 12.3.2018: visibility setting 
        if (use_hide_el == true && el[i].JSPComment.indexOf('attVisible')<0 /*&& el[i].Opts.indexOf('traced')<0*/ && !(el[i].CType == 'measurements' || el[i].CType == 'buttons' || jsp_macros.indexOf(el[i].Type)<0) || el[i].JsxType == 'macros') { 
            
            if (evas[el[i].Row].indexOf('visible:false,')>=0) {
              
              evas[el[i].Row] = evas[el[i].Row].replace(evas[el[i].Row].slice(evas[el[i].Row].indexOf('visible'), evas[el[i].Row].indexOf(',', evas[el[i].Row].indexOf('visible'))+1), '');
              evas[el[i].Row] = evas[el[i].Row] + elid+"_el["+el[i].Row+"].hideElement();";
            }
            else if (evas[el[i].Row].indexOf('visible:false}')>=0) {

              evas[el[i].Row] = evas[el[i].Row].replace(evas[el[i].Row].slice(evas[el[i].Row].indexOf('visible'), evas[el[i].Row].indexOf('}', evas[el[i].Row].indexOf('visible'))), '');
              evas[el[i].Row] = evas[el[i].Row] + elid+"_el["+el[i].Row+"].hideElement();"; }
          } //end update
          else console.log(evas[el[i].Row]);

        if (use_hide_el == true && el[i].JSPComment.indexOf('attVisible')<0 && el[i].Type == 'VectorTranslation' && evas[el[i].Row].indexOf('visible:false,')>0) {
          evas[el[i].Row] = evas[el[i].Row].replace(evas[el[i].Row].slice(evas[el[i].Row].indexOf('visible'), evas[el[i].Row].indexOf(',', evas[el[i].Row].indexOf('visible'))+1), '');
          evas[el[i].Row] = evas[el[i].Row] + elid+"_el["+el[i].Row+"].hideElement();";
        }
      } //end of else

      



      //if (el[i].JSPComment.indexOf('latex')>=0) alert(evas[el[i].Row]);

      //CONVERT ROW            
        eval(evas[el[i].Row]); 

        //alert(evas[el[i].Row]);


      tmpstring = "";
      if (document.getElementById('originalcode').value == 'Comment' && !isFullPageConvert)
        codestring = codestring + "\n /* " +getOrigString(el[i], 0, 0, false, 'SketchrowOnly')+ " */\n";

      if (el[i].comment.length>0 || el[i].JSPComment.length>0 || el[i].LastComment.length>0 ) {
        
        try {      
          //add normal comment if there is one
          if (el[i].comment.length>1) 
            codestring = codestring + el[i].comment + "\n" ; 
          //add new jsx code string
          codestring = codestring + evas[el[i].Row] + "\n";
          //add dot command if there is one
          if (el[i].JSPComment.length>1)
            codestring = codestring + "/* " + el[i].JSPComment.slice(0, this.length-1) + " */\n";
          //finally add last comment if there is one at the end of the code
          if (el[i].LastComment.length > 1)
            codestring = codestring + el[i].LastComment + "\n";

        } catch(e) {alert(el[i].comment);}
      
      }
      else
        codestring = codestring + evas[el[i].Row] + "\n" ;


      if (!isFullPageConvert){
        codestring = codestring+"\n";
      }

  } //end of for

  if (board_update_events.length > 0) {
    board_update_events = boardname+"_b.on('update', function(){" + board_update_events + "});\n\n";
    codestring = codestring + board_update_events;
    eval(board_update_events);

  }

  if (sketch == 'null') errors = errors + document.getElementById('errormessages_prog').innerHTML;

  //Check for errors
   if (errors.length>0 && document.getElementById(code_input) && document.getElementById(code_input).value.length>0) document.getElementById('ERRORS').style.display="block";
  } //end of try
  catch(err){
    document.getElementById('ERRORS').style.display="inline-block";
    errors = errors + "<br>ERROR OCCURRED DURING COMPILATION!<br>Error name: "+err.name+"<br>Error message: "+err.message+"<br>Error stack: "+err.stack+"<br><br>Operation before line "+evas.length +" caused this problem.";
    //document.getElementById('errormessages_dev').innerHTML = "<form action='posterror.php' method='POST' name='Error occurred in JSPtoJSXG'><br>"+"<textarea rows='3' cols='30' name='errormes' style='display:none;'>"+errors+"<"+"/"+"textarea><br>"+"<textarea rows='3' cols='30' name='jspcode' style='display:none;'>"+document.getElementById(code_input).value+"<"+"/"+"textarea><br>"+"<textarea rows='3' cols='40' name='comms'>"+"your own comments here..."+"<"+"/"+"textarea><br>"+"<input type='submit' value='Send errormessage' /"+"></"+"form>";
    document.getElementById('err_mes').value = errors;
    document.getElementById('err_comms').value = "write comments (optional)";
    //document.getElementById('err_jspcode').value = document.getElementById(code_input);
    document.getElementById('err_comms').style.display = "block";
    document.getElementById('err_info').style.display = "block";
    document.getElementById('err_submit_btn').style.display = "block";
  }
        
  // stop the clock
  endingtime = new Date().getTime();

  //calculate time used in convertion
  executiontime = (endingtime - startingtime) / 1000;

  codestring = codestring + "})();//]]>\n<"+"/"+"script"+">"; // end of the converted codetext

  var add_mathjax_button = "";
  /*if (document.getElementById('latex').checked == true || document.getElementById('latex_fixedtext').checked == true) 
    //MathJax.Hub.Queue(["Typeset",MathJax.Hub]);
    add_mathjax_button = '<input onclick="runMathJax()" value="Run MathJax Hub" type="button" class="browseButton">';
  */
  /*if (isFullPageConvert) {
    document.getElementById('jsxgimage').innerHTML = "";
    if (errors.length > 30 || sketch == 'null') document.getElementById('errormessages').innerHTML = "<font color='red'><b>Convertion done in "+executiontime+" seconds, but some errors occurred.<br><br></b></font>"+errors;
    else document.getElementById('errormessages').innerHTML = "<font color='green'><b>Convertion done in "+executiontime+" seconds, no errors occurred.</b></font>";
    return codestring;
  }

  else {*/
    //codestring = "<div><div style='display:inline-block; letter-spacing:1.5px;'>"+add_mathjax_button+"<div style='display:inline-block;margin-right:20px;'><font size='4'><b>JSXGraph code:</b></font></div></div> <div style='display:inline-block; '> <input onclick='downloadJSX()' value='Save JSX/HTML' type='button' class='browseButton'></div> <div style='display:inline-block; '> <input onclick='editJSXcode(1)' value='Open this in JSX code-editor' type='button' class='browseButton' /></div> <div style='display:inline-block; '> <input onclick='copyToClipboard()' value='Copy JSX code to clipboard' type='button' class='browseButton'></div></div><div style='display:block; margin-top:2px;'><textarea id ='jsxcode' cols='' rows='' value="+codestring+" </textarea></div>";
    codestring = add_mathjax_button+"<textarea id ='jsxcode' cols='' rows='' value="+codestring+" </textarea>";
    right_side_titles_and_buttons = document.getElementsByClassName('right_side_hidden_element');
    document.getElementById("jsxgcode").innerHTML = codestring;
    if (errors.length > 30 || sketch == 'null') document.getElementById('errormessages').innerHTML = "<font color='red'><b>Convertion done in "+executiontime+" seconds, but some errors occurred.<br><br></b></font>"+errors;
    else document.getElementById('errormessages').innerHTML = "<font color='green'><b>Convertion done in "+executiontime+" seconds, no errors.</b></font>";
    try {
      if (!isFullPageConvert) {
        for(var i=0; i<right_side_titles_and_buttons.length; i++)
          right_side_titles_and_buttons[i].style.display = "block";
      
        document.getElementById('jsxcode').style.width = "100%";
        document.getElementById('jsxcode').style.height = window.screen.availHeight/2.5 +"px";
        document.getElementById('JSketchPadCode').style.width = "100%";
        document.getElementById('JSketchPadCode').style.height = window.screen.availHeight/2.5 +"px";
      } 
    } catch(e) {alert(e.message)}
    
    codestring = codestring.slice(codestring.indexOf('cript src =')-2, codestring.lastIndexOf('</script')+9);

    //run mathjax hub automatically
    if (document.getElementById('latex').checked || document.getElementById('latex_fixedtext').checked || codestring.indexOf('\\\\')>=0) {
      MathJax.Hub.Queue(["Typeset",MathJax.Hub]);
    }

    return codestring;
  //} 
  /*else {
    document.getElementById("jsxgcode_title_and_buttons").innerHTML = document.getElementById("jsxgcode_title_and_buttons").innerHTML + add_mathjax_button;
    //codestring = "<div><div style='display:inline-block; letter-spacing:1.5px;'>"+add_mathjax_button+"<div style='display:inline-block;margin-right:20px;'><font size='4'><b>JSXGraph code:</b></font></div></div> <div style='display:inline-block; '> <input onclick='downloadJSX()' value='Save JSX/HTML' type='button' class='browseButton'></div> <div style='display:inline-block; '> <input onclick='editJSXcode(1)' value='Open this in JSX code-editor' type='button' class='browseButton' /></div> <div style='display:inline-block; '> <input onclick='copyToClipboard()' value='Copy JSX code to clipboard' type='button' class='browseButton'></div></div><div style='display:block; margin-top:2px;'><textarea id ='jsxcode' cols='' rows='' value="+codestring+" </textarea></div>";
    document.getElementById("jsxgcode_textarea").value = codestring;
    if (errors.length > 30) document.getElementById('errormessages').innerHTML = "<font color='red'><b>Convertion done in "+executiontime+" seconds, but some errors occurred.<br><br></b></font>"+errors;
    else document.getElementById('errormessages').innerHTML = "<font color='green'><b>Convertion done in "+executiontime+" seconds, no errors.</b></font>";
    return codestring;
  }*/
}


