/**  
 * Changes property JsxType of an element given as object created in sketchpadrowToJsxgElement(string)
 * [mostly just changes property to lowercase letters]
 * Also change the CType (constructor type), possible types are:
 * points, straightlines, circles, measurements, transformations, images, buttons, coordinatesystem, locus
 * @method changeTypeToJSX
 * @param {Object} a - 
 * @return {Object} a - return the object
 */  
function changeTypeToJSX(a) {
  try {
    if (a.Type == "FixedPoint") {
        a.JsxType = 'point'; 
        a.Opts = a.Opts.replace("]",",fixed:true]"); 
        a.CType = 'points';
    } 
    else if (a.Type == "Point" || a.Type == "PlotXY" || a.Type == "UnitPoint") {
      a.JsxType = a.Type.toLowerCase();
      a.CType = 'points';
    }
    else if (a.Type == "Origin&Unit" || a.Type == "OriginAndUnit" || a.Type == "Origin&amp;Unit") {
      a.JsxType = 'originandunit';
      a.CType = 'coorninatesystem';
      a.ParentType = 'point';
    }
    else if (a.Type == "Point on object") {
      a.JsxType = 'glider';
      a.CType = 'points';
      a.ParentType = 'line';
    }
    else if (a.Type == "MidPoint" || a.Type == "Midpoint") {
      a.JsxType = a.Type.toLowerCase();
      a.CType = 'points';
      a.ParentType = 'line';
    }
    else if (a.Type=='Line' || a.Type == 'Segment' || a.Type == 'Ray' || a.Type == 'Parallel' || a.Type == 'Perpendicular' || a.Type == 'AxisX' || a.Type == 'AxisY') {
      a.JsxType = a.Type.toLowerCase();
      a.CType = 'straightlines';
      a.ParentType = 'point';
    }
    else if (a.Type == 'Circle' || a.Type == 'Circle by radius' ) {
      a.JsxType = a.Type.toLowerCase();
      a.CType = 'circles';
      a.ParentType = 'point';
    }
    else if (a.Type == "FixedText") {
      a.JsxType = 'text';
      a.CType = 'buttons';
    }
    else if (a.Type == "Intersect1" || a.Type == "Intersect2" || a.Type == "Intersect") {
      if (a.Type == "Intersect2") a.Att.Info = 0; else a.Att.Info = 1;
        a.JsxType = 'intersection'; 
        a.CType = 'points';
        a.ParentType = 'line';
      }
    else if ( a.Type == 'Reflection' || a.Type == 'Dilation' || a.Type == 'Dilation/SegmentRatio' || a.Type == 'Dilation/3ptRatio' || a.Type == 'Dilation/3PtRatio' || 
              a.Type == 'Dilation/MarkedRatio' || a.Type == 'Rotation' || a.Type == 'Rotation/MarkedAngle' || a.Type == 'Rotation/MeasuredAngle'  || 
              a.Type == 'Translation' || a.Type == 'VectorTranslation' || a.Type == 'Translation/MarkedAngle/MarkedDistance' || 
              a.Type == 'Translation/Markedangle/Markeddistance' || a.Type == 'PolarTranslation' || a.Type == 'Translation/FixedAngle/MarkedDistance') {
      a.JsxType = a.Type.toLowerCase();
      a.CType = 'transformations';

    }
    else if ( a.Type == 'Coordinates' || a.Type == 'Length' || a.Type == 'Angle' || a.Type == 'Perimeter' || a.Type == 'Circumference' || 
              a.Type == 'Radius' || a.Type == 'Area' || a.Type == 'Slope' || a.Type == 'Distance' || a.Type == 'Ratio/Segments' || 
              a.Type == 'Ratio/Points' || a.Type == 'Calculate' || a.Type == 'Parameter') {
      a.JsxType = a.Type.toLowerCase();
      a.CType = 'measurements'
    }
    else if (a.Type == 'ShowButton' || a.Type == 'HideButton' || a.Type == 'SimultaneousButton' || a.Type == 'AnimateButton' || a.Type == 'MoveButton') {
      a.JsxType = a.Type.toLowerCase();
      a.CType = 'buttons';
    }
    else if (a.Type == 'Polygon' ) {
      a.JsxType = 'polygon';
      a.CType = 'interiors';
    }
    else if (a.Type == 'Circle interior' ) {
      a.JsxType = a.Type.toLowerCase();
      a.CType = 'interiors';
    }
    else if (a.Type == 'Locus') {
      a.JsxType = a.Type.toLowerCase();
      a.CType='curves';
    }
    else if (jsp_macros.indexOf(a.Type)>=0)
      a.JsxType = 'macros';
    else {
      a.JsxType = a.Type.toLowerCase();
      a.CType = 'unknown';
      //alert(a.JsxType);
    }
  } catch(e) {alert(e.message);}
  return a;
    
}

/**
 * Changes property Pos of an element given as object created in sketchpadrowToJsxgElement(string)
 * Outcome depends of type of the element as follows:
 * - Point will change Pos to array with point coordinates [x,y],
 * - Unit coord will change Pos to array with reference to object and a coordinate [ref to obj, y],
 * @method changePosInfoToJSX
 * @param {Object} a -
 * @param {Array} applets -
 * @param {Int} coordd - coord divider, this is read from the original code in the main function JSPtoSJX
 * @param {Double} scx - scale x factor, this is read in the main function
 * @param {Double} scy - scale y factor, this is read in the main function
 * @param {Int} hasoriginunit - origin&unit info read in the main function from the original code
 * @return {Object} a - return the object with changed values
 * ... 
 */  
function changePosInfoToJSX(a, applets, coordd, scx, scy, hasoriginunit) {

  var x,y,o,t,values = [], laba, butvadj=0;
    //button vertical adjustment:
    if (document.getElementById('usebuttonstyle').checked == true) butvadj = document.getElementById('buttonverticaladjsel').value;

    x = a.Pos.slice(0,a.Pos.indexOf(","));
    y = a.Pos.slice(a.Pos.indexOf(",")+1,a.Pos.length);
    
    if (a.Pos.indexOf('#-1#')>=0 || a.Pos.indexOf('#-2#')>=0 || a.Pos.indexOf('#-3#')>=0|| a.Pos.indexOf('#-4#')>=0) {
      while (a.Pos.indexOf('#-1#')>=0)
        a.Pos = a.Pos.replace('#-1#', a.Row-1);
      while (a.Pos.indexOf('#-2#')>=0)
        a.Pos = a.Pos.replace('#-2#', a.Row-2);
      while (a.Pos.indexOf('#-3#')>=0)
        a.Pos = a.Pos.replace('#-3#', a.Row-3);
      while (a.Pos.indexOf('#-4#')>=0)
        a.Pos = a.Pos.replace('#-4#', a.Row-4);
    }

    if (a.JsxType == 'point')
      a.Pos = changeToCoord(x,y,applets,coordd,scx,scy,hasoriginunit);
    else if (a.JsxType == 'unitpoint'){
      a.Pos = [eval(x), changeToCoord(x,y,applets,coordd,scx,scy,hasoriginunit)[1]]; 
    }
    else if (a.JsxType == 'originandunit') {
      a.Pos = eval("["+a.Pos+"]");
    }
    else if (a.JsxType == 'text' || a.JsxType == 'image') {
      y = a.Pos.slice(a.Pos.indexOf(",")+1,a.Pos.indexOf(",", a.Pos.indexOf(",")+1));
      t = a.Pos.slice(a.Pos.indexOf("'")+1,a.Pos.lastIndexOf("'"));
      t = cleanString(t);

      if (a.JSPComment.indexOf('text')>=0) t = a.JSPComment.slice(a.JSPComment.indexOf("'", a.JSPComment.indexOf('text'))+1, a.JSPComment.lastIndexOf("'", bracketClosure(a.JSPComment, '(', 'text')));
      if (a.JSPComment.indexOf('latex')>=0) t = '\\\\"+"( ' + a.JSPComment.slice(a.JSPComment.indexOf('(', a.JSPComment.indexOf("latex"))+1, bracketClosure(a.JSPComment, "(", "latex")-1) + ' \\\\"+")';
      t=subAndSup(t);
      t=changeTextApostrophes(t);
      //alert(t.length);
      if (a.JSPComment.indexOf('LabelAlign')>=0 || a.JSPComment.indexOf('labelAlign')>=0) {laba = eval(a.Att.LabelAlign.slice(a.Att.LabelAlign.indexOf('['), a.Att.LabelAlign.indexOf(']')+1));  x = eval(x)+eval(laba[0]); y = eval(y) + eval(laba[1]);}
      
      a.Pos = [changeToCoord(x,y,applets,coordd,scx,scy,hasoriginunit)[0],changeToCoord(x,y,applets,coordd,scx,scy,hasoriginunit)[1],t];
    }
    //else if (a.JsxType == 'vectortranslation') {
      //alert(a.Pos);
    //  }
    else if (a.JsxType == 'movebutton') {
      y = eval(a.Pos.slice(a.Pos.indexOf(",")+1,a.Pos.indexOf(",", a.Pos.indexOf(",")+1))) + eval(butvadj);
      o = 1000 * eval(a.Pos.slice(a.Pos.indexOf(",", a.Pos.indexOf(",")+1)+1, a.Pos.indexOf(",'")));     
        t = a.Pos.slice(a.Pos.indexOf("'")+1, a.Pos.lastIndexOf("'"));
        if (a.JSPComment.indexOf('text')>=0) t = a.JSPComment.slice(a.JSPComment.indexOf("'", a.JSPComment.indexOf('label'))+1, a.JSPComment.lastIndexOf("'", bracketClosure(a.JSPComment, '(', 'text')));
        t=subAndSup(t);
        t=changeTextApostrophes(t);
      a.Pos = [changeToCoord(x,y,applets,coordd,scx,scy,hasoriginunit)[0],changeToCoord(x,y,applets,coordd,scx,scy,hasoriginunit)[1],o,t]; 
    }
    else if (a.JsxType == 'showbutton' || a.JsxType == 'hidebutton' || a.JsxType == 'simultaneousbutton' || a.JsxType == 'animatebutton') {
      y = eval(a.Pos.slice(a.Pos.indexOf(",")+1,a.Pos.indexOf(",", a.Pos.indexOf(",")+1))) + eval(butvadj); 
      t = a.Pos.slice(a.Pos.indexOf("'")+1,a.Pos.lastIndexOf("'")); 
        if (t.indexOf('F(')>=0) { t=t.replace(/F/g, '&#70;'); }
      if (t.length==0)  t="UNLABELED BUTTON";
      if (a.JSPComment.indexOf('text')>=0) t = a.JSPComment.slice(a.JSPComment.indexOf("'", a.JSPComment.indexOf('label'))+1, a.JSPComment.lastIndexOf("'", bracketClosure(a.JSPComment, '(', 'text')));
      t=subAndSup(t);
      t=changeTextApostrophes(t);
      //alert(t);
      a.Pos = [changeToCoord(x,y,applets,coordd,scx,scy,hasoriginunit)[0], changeToCoord(x,y,applets,coordd,scx,scy,hasoriginunit)[1], t];
    }
    else if (a.JsxType == 'ratio/points' || a.JsxType == 'angle') {
        values = eval("["+a.Pos.slice(0,a.Pos.lastIndexOf(",", a.Pos.indexOf("'")))+"]");
        values[3] = changeToCoord(values[3],values[4],applets,coordd,scx,scy,hasoriginunit)[0];
        values[4] = changeToCoord(values[3],values[4],applets,coordd,scx,scy,hasoriginunit)[1];
        
        if (a.JSPComment.indexOf('text')>=0) 
          t = a.JSPComment.slice(a.JSPComment.indexOf("'", a.JSPComment.indexOf('text'))+1, a.JSPComment.lastIndexOf("'", bracketClosure(a.JSPComment, '(', 'text')));
        else if (a.JSPComment.indexOf('latex')>=0) t = "\\\\'+'( " + a.JSPComment.slice(a.JSPComment.indexOf("(", a.JSPComment.indexOf('latex'))+1, bracketClosure(a.JSPComment, '(', 'latex')-1) + " \\\\'+')";
        
        else 
          t = (a.Pos.slice(a.Pos.indexOf("'")+1,a.Pos.lastIndexOf("'")));
        
        t=subAndSup(t);
        t=changeTextApostrophes(t);
        values.push(t);
        if (a.JSPComment.indexOf('LabelAlign')>=0) {laba = eval(a.Att.LabelAlign.slice(a.Att.LabelAlign.indexOf('['), a.Att.LabelAlign.indexOf(']')+1));  values[3] = eval(values[3]+laba[0]); values[4] = eval(values[4] + laba[1]);}
        a.Pos = values; 
    }
    else if (a.JsxType == 'distance' || a.JsxType == 'coordinates' || a.JsxType == 'ratio/segments') {
      values = eval("["+a.Pos.slice(0,a.Pos.lastIndexOf(",", a.Pos.indexOf("'")))+"]");
        values[2] = changeToCoord(values[2],values[3],applets,coordd,scx,scy,hasoriginunit)[0];
        values[3] = changeToCoord(values[2],values[3],applets,coordd,scx,scy,hasoriginunit)[1];
        
        if (a.JSPComment.indexOf('text')>=0) 
          t = a.JSPComment.slice(a.JSPComment.indexOf("'", a.JSPComment.indexOf('text'))+1, a.JSPComment.lastIndexOf("'", bracketClosure(a.JSPComment, '(', 'text')));
        else if (a.JSPComment.indexOf('latex')>=0) t = "\\\\'+'( " + a.JSPComment.slice(a.JSPComment.indexOf("(", a.JSPComment.indexOf('latex'))+1, bracketClosure(a.JSPComment, '(', 'latex')-1) + " \\\\'+')";
      
        else 
          t = (a.Pos.slice(a.Pos.indexOf("'")+1,a.Pos.lastIndexOf("'")));
        
        t=subAndSup(t);
        t=changeTextApostrophes(t);
        values.push(t);
        if (a.JSPComment.indexOf('LabelAlign')>=0) {laba = eval(a.Att.LabelAlign.slice(a.Att.LabelAlign.indexOf('['), a.Att.LabelAlign.indexOf(']')+1));  values[3] = eval(values[3]+laba[0]); values[4] = eval(values[4] + laba[1]);}

        a.Pos = values; 
    }
    else if (a.JsxType == 'length' || a.JsxType == 'circumference' || a.JsxType == 'radius' || a.JsxType == 'area' || a.JsxType == 'slope') {
      values = eval("["+a.Pos.slice(0,a.Pos.lastIndexOf(",", a.Pos.indexOf("'")))+"]");
      values[1] = changeToCoord(values[1],values[2],applets,coordd,scx,scy,hasoriginunit)[0];
      values[2] = changeToCoord(values[1],values[2],applets,coordd,scx,scy,hasoriginunit)[1];
      
      if (a.JSPComment.indexOf('text')>=0) 
        t = a.JSPComment.slice(a.JSPComment.indexOf("'", a.JSPComment.indexOf('text'))+1, a.JSPComment.lastIndexOf("'", bracketClosure(a.JSPComment, '(', 'text')));
      else if (a.JSPComment.indexOf('latex')>=0) t = "\\\\'+'( " + a.JSPComment.slice(a.JSPComment.indexOf("(", a.JSPComment.indexOf('latex'))+1, bracketClosure(a.JSPComment, '(', 'latex')-1) + " \\\\'+')";
    
      else 
        t = (a.Pos.slice(a.Pos.indexOf("'")+1,a.Pos.lastIndexOf("'")));
      
      t=subAndSup(t);
      t=changeTextApostrophes(t);
      values.push(t);
      if (a.JSPComment.indexOf('LabelAlign')>=0) {laba = eval(a.Att.LabelAlign.slice(a.Att.LabelAlign.indexOf('['), a.Att.LabelAlign.indexOf(']')+1));  values[3] = eval(values[3]+laba[0]); values[4] = eval(values[4] + laba[1]);}

      a.Pos = values;
    }
    else if (a.JsxType == 'calculate') {
      values = eval("["+ a.Pos.slice(0,a.Pos.indexOf(",", a.Pos.indexOf(",")+1)) + "]");
      values = changeToCoord(values[0],values[1],applets,coordd,scx,scy,hasoriginunit);
        
        if (a.JSPComment.indexOf('text')>=0) 
          values[2] = a.JSPComment.slice(a.JSPComment.indexOf("'", a.JSPComment.indexOf('text'))+1, a.JSPComment.lastIndexOf("'", bracketClosure(a.JSPComment, '(', 'text')));
        else if (a.JSPComment.indexOf('latex')>=0) values[2] = "\\\\'+'( " + a.JSPComment.slice(a.JSPComment.indexOf("(", a.JSPComment.indexOf('latex'))+1, bracketClosure(a.JSPComment, '(', 'latex')-1) + " \\\\'+')";
      
        else {
          values[2] = a.Pos.slice(a.Pos.indexOf("'")+1, a.Pos.lastIndexOf("'", a.Pos.lastIndexOf("'")-1));
          values[2] = values[2].slice(0, values[2].lastIndexOf("'"));
          values[2] = cleanString(values[2]);
        }
        values[2]=subAndSup(values[2]);
        values[2]=changeTextApostrophes(values[2]);
        values[3] = a.Pos;
        values[4] = eval("["+a.Pos.slice(a.Pos.lastIndexOf("(")+1,a.Pos.lastIndexOf(")"))+"]");
        
        if (a.JSPComment.indexOf('LabelAlign')>=0) {laba = eval(a.Att.LabelAlign.slice(a.Att.LabelAlign.indexOf('['), a.Att.LabelAlign.indexOf(']')+1));  values[0] = eval(values[0]+laba[0]); values[1] = eval(values[1] + laba[1]);}

        a.Pos = values;
    }
    //else if (a.JsxType == 'circle interior') {

    //}
    else {
      //else if (a.JsxType == 'dilation/markedratio' || a.JsxType == 'line' || a.JsxType == 'segment' || a.JsxType == 'circle' || a.JsxType == 'circle by radius' || a.JsxType == 'perpendicular' || a.JsxType == 'intersection' || a.JsxType == 'glider' || a.JsxType == 'ray' || a.JsxType == 'reflection' || a.JsxType == 'midpoint' || a.JsxType == 'parallel' || a.JsxType == 'axisx' || a.JsxType == 'axisy' || a.JsxType == 'vectortranslation' || a.JsxType == 'translation' || a.JsxType == 'dilation/3ptratio') 
      
      a.Pos = eval("["+a.Pos+"]"); 
    } 

    return a;
}

function getElemRefs(el) {
  try {
    if (el.Type == 'Point on object' || el.Type == 'UnitPoint' || el.Type == 'Dilation' || el.Type == 'Rotation' ||  el.Type == 'Translation/FixedAngle/MarkedDistance' ||  el.Type == 'Translation/MarkedAngle/FixedDistance' || el.Type == 'Locus') {
      el.Ref = eval('[' + el.Pos.slice(0, this.lastIndexOf(',')) + ']');
    }
    else if (el.Type == 'Translation' || el.Type == 'PolarTranslation' ) {
      el.Ref = eval('[' + el.Pos.slice(0, this.lastIndexOf(',', this.lastIndexOf(','))) + ']');
    }
    else if (el.CType == 'measurements') {
      
      if (el.Type == 'Angle' || el.Type == 'Ratio/Points') //has 3 objRef (Angle, Ratio/Points)
        el.Ref = eval('[' + el.Pos.slice(0, this.indexOf(',', this.indexOf(',', this.indexOf(',')+1)+1)) + ']');
      else if (el.Type == 'Distance' || el.Type == 'Ratio/Segments') //has 2 objRef (Distance, Ratio/Segments, )
        el.Ref = eval('[' + el.Pos.slice(0, this.indexOf(',', this.indexOf(',')+1)) + ']');      
      else if (el.Type == 'Calculate') //is calculate
        el.Ref = eval('[' + el.Pos.slice(this.lastIndexOf('(')+1, this.length-1) + ']');
      else //has 1 objRef (Length, Perimeter, Circumference, Radius, Area, Slope, Coordinates, Parameter)
        el.Ref = eval('[' + el.Pos.slice(0, this.indexOf(',')) + ']');
    }
    else if (el.CType == 'buttons') {
      el.Ref = eval('[' + el.Pos.slice(this.lastIndexOf('(')+1, this.length) + ']');
    }
    else if (el.Type == 'FixedText' || el.Type == 'FixedPoint' || el.Type == 'Point' || el.Type == 'Image')
      el.Ref = -1;
    else {
      el.Ref = eval('[' + el.Pos + ']');
    }
  } catch(err) {alert("Error: Line " + el.Row + ", " +err.message);}
  return el;
}